from __future__ import annotations

import hashlib
import itertools
import json
import logging
import secrets
import tempfile
import time
import warnings
from typing import Literal, get_args
from urllib.parse import urlencode

import cachetools
import gradio as gr
import pandas as pd

import mteb
from mteb import BenchmarkResults
from mteb.benchmarks.benchmark import RtebBenchmark
from mteb.cache import ResultCache
from mteb.leaderboard.benchmark_selector import (
    DEFAULT_BENCHMARK_NAME,
    GP_BENCHMARK_ENTRIES,
    R_BENCHMARK_ENTRIES,
    _make_selector,
)
from mteb.leaderboard.event_logger import EventLogger
from mteb.leaderboard.figures import (
    _performance_over_time_plot,
    _performance_size_plot,
    _radar_chart,
)
from mteb.leaderboard.table import (
    apply_per_language_styling_from_benchmark,
    apply_per_task_styling_from_benchmark,
    apply_summary_styling_from_benchmark,
)
from mteb.leaderboard.text_segments import ACKNOWLEDGEMENT, FAQ
from mteb.models.model_meta import MODEL_TYPES

logger = logging.getLogger(__name__)
event_logger = EventLogger()


LANGUAGE: list[str] = list({l for t in mteb.get_tasks() for l in t.metadata.languages})
MODEL_TYPE_CHOICES = list(get_args(MODEL_TYPES))


def _produce_benchmark_link(benchmark_name: str, request: gr.Request) -> str:
    """Produces a URL for the selected benchmark.

    Returns:
        A markdown string containing the URL.
    """
    params = urlencode(
        {
            "benchmark_name": benchmark_name,
        }
    )
    base_url = request.request.base_url
    md = "You can also share this benchmark using the following link:\n"
    url = f"{base_url}?{params}"
    md += f"```\n{url}\n```"
    return md


def _set_benchmark_on_load(request: gr.Request):
    query_params = request.query_params
    return query_params.get("benchmark_name", DEFAULT_BENCHMARK_NAME)


def _download_table(table: pd.DataFrame) -> str:
    file = tempfile.NamedTemporaryFile(delete=False, suffix=".csv")
    table.to_csv(file)
    return file.name


def _update_citation(benchmark_name: str) -> str:
    benchmark = mteb.get_benchmark(benchmark_name)
    if benchmark.citation is not None:
        citation = "To cite this work, please use the following reference:\n"
        citation += f"```bibtex\n{benchmark.citation}\n```"
    else:
        citation = ""
    return citation


def _update_description(
    benchmark_name: str, languages: list[str], task_types: list[str], domains: list[str]
) -> str:
    benchmark = mteb.get_benchmark(benchmark_name)
    description = f"{benchmark.description}\n"
    n_languages = len(languages)
    n_task_types = len(task_types)
    n_tasks = len(benchmark.tasks)
    n_domains = len(domains)
    description += f" - **Number of languages**: {n_languages}\n"
    description += f" - **Number of tasks**: {n_tasks}\n"
    description += f" - **Number of task types**: {n_task_types}\n"
    description += f" - **Number of domains**: {n_domains}\n"
    if benchmark.reference is not None:
        description += (
            f'\n<a href="{benchmark.reference}" target="_blank">Click for More Info</a>'
        )

    return description


def _format_list(props: list[str]):
    if props is None:
        return ""
    if len(props) > 3:
        return ", ".join(props[:3]) + "..."
    return ", ".join(props)


def _update_task_info(task_names: str) -> pd.DataFrame:
    tasks = mteb.get_tasks(tasks=task_names)
    df = tasks.to_dataframe(
        properties=[
            "name",
            "type",
            "languages",
            "domains",
            "reference",
            "main_score",
            "modalities",
            "is_public",
        ]
    )
    df["languages"] = df["languages"].map(_format_list)
    df = df.sort_values("name")
    df["domains"] = df["domains"].map(_format_list)
    df["name"] = df.apply(
        lambda row: f'<a href="{row["reference"]}" target="_blank">{row["name"]}</a>',
        axis=1,
    )
    df["modalities"] = df["modalities"].map(_format_list)
    df = df.rename(
        columns={
            "name": "Task Name",
            "type": "Task Type",
            "languages": "Languages",
            "domains": "Domains",
            "main_score": "Metric",
            "modalities": "Modality",
            "is_public": "Public",
        }
    )
    df = df.drop(columns="reference")
    return df


# Model sizes in million parameters
MIN_MODEL_SIZE, MAX_MODEL_SIZE = 0, 100_000


def _filter_models(
    model_names: list[str],
    task_select: list[str],
    availability: bool | None,
    compatibility: list[str],
    instructions: bool | None,
    max_model_size: int,
    zero_shot_setting: Literal["only_zero_shot", "allow_all", "remove_unknown"],
    model_types: list[str] | None,
):
    lower, upper = 0, max_model_size
    # Setting to None, when the user doesn't specify anything
    if (lower == MIN_MODEL_SIZE) or (lower is None):
        lower = None
    else:
        # Multiplying by millions
        lower = lower * 1e6
    if (upper == MAX_MODEL_SIZE) or (upper is None):
        upper = None
    else:
        upper = upper * 1e6
    model_metas = mteb.get_model_metas(
        model_names=model_names,
        open_weights=availability,
        use_instructions=instructions,
        frameworks=compatibility,
        n_parameters_range=(lower, upper),
        model_types=model_types,
    )

    models_to_keep = set()
    for model_meta in model_metas:
        is_model_zero_shot = model_meta.is_zero_shot_on(task_select)
        if is_model_zero_shot is None:
            if zero_shot_setting in ["remove_unknown", "only_zero_shot"]:
                continue
        elif not is_model_zero_shot:
            if zero_shot_setting == "only_zero_shot":
                continue
        models_to_keep.add(model_meta.name)
    return list(models_to_keep)


def _should_show_zero_shot_filter(benchmark_name: str) -> bool:
    benchmark = mteb.get_benchmark(benchmark_name)

    if isinstance(benchmark, RtebBenchmark):
        return False
    return True


@cachetools.cached(
    cache={},
    key=lambda benchmark_name, all_benchmark_results: hash(benchmark_name),
)
def _cache_on_benchmark_select(benchmark_name, all_benchmark_results):
    start_time = time.time()
    benchmark = mteb.get_benchmark(benchmark_name)
    languages = [task.languages for task in benchmark.tasks if task.languages]
    languages = set(itertools.chain.from_iterable(languages))
    languages = sorted(languages)
    domains = [
        task.metadata.domains for task in benchmark.tasks if task.metadata.domains
    ]
    domains = set(itertools.chain.from_iterable(domains))
    types = {task.metadata.type for task in benchmark.tasks if task.metadata.type}
    modalities = set()
    for task in benchmark.tasks:
        modalities.update(task.metadata.modalities)
    languages, domains, types, modalities = (
        sorted(languages),
        sorted(domains),
        sorted(types),
        sorted(modalities),
    )
    elapsed = time.time() - start_time
    benchmark_results = all_benchmark_results[benchmark_name]
    scores = benchmark_results._get_scores(format="long")
    logger.debug(f"on_benchmark_select callback: {elapsed}s")
    show_zero_shot = _should_show_zero_shot_filter(benchmark_name)

    # Calculate initial models for this benchmark to avoid race conditions
    benchmark_tasks = sorted([task.metadata.name for task in benchmark.tasks])
    all_models_in_scores = list({entry["model_name"] for entry in scores})
    initial_models = _filter_models(
        all_models_in_scores,
        benchmark_tasks,
        availability=None,
        compatibility=[],
        instructions=None,
        max_model_size=MAX_MODEL_SIZE,
        zero_shot_setting="allow_all",
        model_types=MODEL_TYPE_CHOICES,
    )
    # Sort to ensure consistency with update_models
    initial_models = sorted(initial_models)

    return (
        languages,
        domains,
        types,
        modalities,
        benchmark_tasks,
        scores,
        show_zero_shot,
        initial_models,
    )


@cachetools.cached(
    cache={},
    key=lambda benchmark_name,
    type_select,
    domain_select,
    lang_select,
    modality_select: hash(
        (
            hash(benchmark_name),
            hash(tuple(type_select)),
            hash(tuple(domain_select)),
            hash(tuple(lang_select)),
            hash(tuple(modality_select)),
        )
    ),
)
def _cache_update_task_list(
    benchmark_name, type_select, domain_select, lang_select, modality_select
):
    if not len(lang_select):
        return []
    start_time = time.time()
    benchmark_tasks = []
    tasks_to_keep = []
    for task in mteb.get_benchmark(benchmark_name).tasks:
        benchmark_tasks.append(task.metadata.name)
        if task.metadata.type not in type_select:
            continue
        if task.metadata.domains and not (
            set(task.metadata.domains) & set(domain_select)
        ):
            continue
        if task.languages and not (set(task.languages) & set(lang_select)):
            continue
        if task.metadata.modalities and not (
            set(task.metadata.modalities) & set(modality_select)
        ):
            continue
        tasks_to_keep.append(task.metadata.name)
    benchmark_tasks.sort()
    tasks_to_keep.sort()
    elapsed = time.time() - start_time
    logger.debug(f"update_task_list callback: {elapsed}s")

    return benchmark_tasks, tasks_to_keep


def _filter_benchmark_results_for_tables(
    benchmark_results: BenchmarkResults,
    task_names: set[str] | None,
    model_names: set[str],
    languages: list[str] | None,
) -> BenchmarkResults:
    """Apply task/model/language filters to BenchmarkResults for table rendering."""
    if not task_names:
        return benchmark_results._filter_tasks(task_names=[])

    filtered_benchmark_results = benchmark_results._filter_tasks(
        task_names=sorted(task_names)
    )

    # Keep only language-compatible subsets for each selected task.
    if languages:
        filtered_tasks = mteb.get_tasks(tasks=sorted(task_names), languages=languages)
        filtered_benchmark_results = filtered_benchmark_results.select_tasks(
            filtered_tasks
        )

    if model_names:
        filtered_benchmark_results = filtered_benchmark_results.select_models(
            sorted(model_names)
        )

    return filtered_benchmark_results


def _generate_fingerprint_session_id(request: gr.Request) -> str:
    """Generate a fallback session ID based on request fingerprint.

    This is used when browser storage is unavailable (degraded mode). The fingerprint
    is generated from IP address and browser characteristics to ensure the same user
    gets a consistent session ID across page refreshes even without browser storage.

    Args:
        request: Gradio request object containing client info and headers

    Returns:
        A fingerprint-based session ID with 'fingerprint_' prefix
    """
    try:
        # Collect browser fingerprint factors including IP
        factors = [
            request.client.host if request.client else "unknown",  # IP address
            request.headers.get("user-agent", "unknown"),  # Browser info
            request.headers.get("accept-language", "unknown"),  # Language preference
        ]

        # Create a hash of the fingerprint
        fingerprint = "|".join(factors)
        hash_value = hashlib.sha256(fingerprint.encode()).hexdigest()[:16]

        return f"fingerprint_{hash_value}"
    except Exception as e:
        # If fingerprint generation fails, generate a temporary random ID
        logger.warning(f"Failed to generate fingerprint session ID: {e}")
        return f"temp_{secrets.token_hex(8)}"


def on_page_load(session_data_json: str, request: gr.Request):
    """Log page view with session tracking.

    Handles two modes based on browser storage availability:
    - Normal mode (localStorage available): Uses random token, persisted in browser
    - Degraded mode (localStorage blocked): Uses fingerprint, regenerated but consistent

    Args:
        session_data_json: JSON string containing session data (with _storage_available flag)
        request: Gradio request object for fingerprint generation

    Returns:
        Tuple of (updated JSON string with session data, session_id string)
    """
    # Parse existing session data
    try:
        session_data = json.loads(session_data_json) if session_data_json else {}
    except json.JSONDecodeError:
        logger.warning(f"Failed to parse session data JSON: {session_data_json}")
        session_data = {}

    # Check storage availability from JavaScript
    storage_available = session_data.pop("_storage_available", False)

    # Get or generate session_id based on storage availability
    session_id = session_data.get("session_id", "")

    if not session_id:
        if storage_available:
            # Normal mode: Generate random token that will be persisted
            session_id = f"browser_{secrets.token_hex(16)}"
            logger.info(f"Normal mode - generated session: {session_id[:20]}...")
        else:
            # Degraded mode: Generate fingerprint-based ID for consistency
            session_id = _generate_fingerprint_session_id(request)
            logger.warning(f"Degraded mode - using fingerprint: {session_id[:20]}...")

    # Update session data
    session_data.update(
        {
            "session_id": session_id,
            "storage_available": storage_available,
            "last_visit": time.time(),
        }
    )

    # Log the page view
    event_logger.log_page_view(
        session_id=session_id,
        benchmark=None,
    )

    # Return updated JSON string and extracted session_id
    return json.dumps(session_data), session_id


def get_leaderboard_app(
    cache: ResultCache = ResultCache(), rebuild: bool = False
) -> gr.Blocks:
    """Returns a Gradio Blocks app for the MTEB leaderboard.

    Args:
        cache: ResultCache instance for managing benchmark data
        rebuild: If True, bypasses any pre-computed JSON cache and forces a full rebuild
                from the results repository. This clones/pulls the full results git repo,
                builds BenchmarkResults from individual model files, and overwrites the
                existing cache. Useful for ensuring the freshest data or debugging cache issues.

    Returns:
        gr.Blocks: A Gradio Blocks application configured with the MTEB leaderboard interface
    """
    app_start = time.time()
    logger.info("=== Starting leaderboard app initialization ===")

    logger.info("Step 1/7: Loading all benchmark results...")
    load_start = time.time()
    all_results = cache._load_from_cache(rebuild=rebuild)
    load_time = time.time() - load_start
    logger.info(f"Step 1/7 complete: Loaded results in {load_time:.2f}s")

    logger.info("Step 2/7: Fetching benchmarks...")
    bench_start = time.time()
    benchmarks = sorted(
        mteb.get_benchmarks(display_on_leaderboard=True), key=lambda x: x.name
    )
    bench_time = time.time() - bench_start
    logger.info(
        f"Step 2/7 complete: Fetched {len(benchmarks)} benchmarks in {bench_time:.2f}s"
    )

    logger.info(
        "Step 3/7: Processing all benchmarks (select_tasks + join_revisions)..."
    )
    process_start = time.time()
    all_benchmark_results = {
        benchmark.name: all_results.select_tasks(benchmark.tasks).join_revisions()
        for benchmark in benchmarks
    }
    process_time = time.time() - process_start
    if len(benchmarks) > 0:
        logger.info(
            f"Step 3/7 complete: Processed {len(benchmarks)} benchmarks in {process_time:.2f}s (avg {process_time / len(benchmarks):.2f}s/benchmark)"
        )
    else:
        logger.info(
            f"Step 3/7 complete: Processed 0 benchmarks in {process_time:.2f}s (avg N/A)"
        )

    default_benchmark = mteb.get_benchmark(DEFAULT_BENCHMARK_NAME)
    default_results = all_benchmark_results[default_benchmark.name]
    default_task_types = {
        task_type
        for task_type in default_results.task_types
        if task_type != "InstructionRetrieval"
    }
    display_radar_chart = len(default_task_types) > 1

    logger.info("Step 4/7: Filtering models...")
    filter_start = time.time()

    default_scores = default_results._get_scores(format="long")
    all_models = list({entry["model_name"] for entry in default_scores})
    filtered_models = _filter_models(
        all_models,
        default_results.task_names,
        availability=None,
        compatibility=[],
        instructions=None,
        max_model_size=MAX_MODEL_SIZE,
        zero_shot_setting="allow_all",
        model_types=MODEL_TYPE_CHOICES,
    )
    default_filtered_scores = [
        entry for entry in default_scores if entry["model_name"] in filtered_models
    ]

    # Filter BenchmarkResults based on default filtered models (as required by Kenneth)
    filtered_model_names = [entry["model_name"] for entry in default_filtered_scores]
    filtered_benchmark_results = default_results.select_models(filtered_model_names)
    filter_time = time.time() - filter_start
    logger.info(
        f"Step 4/7 complete: Filtered {len(filtered_model_names)} models in {filter_time:.2f}s"
    )

    logger.info("Step 5/7: Generating tables...")
    table_start = time.time()
    summary_table = apply_summary_styling_from_benchmark(
        default_benchmark, filtered_benchmark_results
    )
    per_task_table = apply_per_task_styling_from_benchmark(
        default_benchmark, filtered_benchmark_results
    )
    per_language_table = apply_per_language_styling_from_benchmark(
        default_benchmark,
        filtered_benchmark_results,
    )
    table_time = time.time() - table_start
    logger.info(f"Step 5/7 complete: Generated tables in {table_time:.2f}s")

    # Check if this benchmark displays per-language results
    display_language_table = len(default_benchmark.language_view) > 0

    logger.info("Step 6/7: Creating Gradio components...")
    component_start = time.time()
    lang_select = gr.CheckboxGroup(
        sorted(default_results.languages),
        value=sorted(default_results.languages),
        show_label=True,
        show_select_all=True,
        label="Language",
        info="Select languages to include.",
    )
    type_select = gr.CheckboxGroup(
        sorted(default_results.task_types),
        value=sorted(default_results.task_types),
        show_label=True,
        show_select_all=True,
        label="Task Type",
        info="Select task types to include.",
    )
    domain_select = gr.CheckboxGroup(
        sorted(default_results.domains),
        value=sorted(default_results.domains),
        show_label=True,
        show_select_all=True,
        label="Domain",
        info="Select domains to include.",
    )
    task_select = gr.CheckboxGroup(
        sorted(default_results.task_names),
        value=sorted(default_results.task_names),
        show_label=True,
        show_select_all=True,
        label="Task",
        info="Select specific tasks to include",
    )
    modality_select = gr.CheckboxGroup(
        sorted(default_results.modalities),
        value=sorted(default_results.modalities),
        show_label=True,
        show_select_all=True,
        label="Modality",
        info="Select modalities to include.",
    )
    component_time = time.time() - component_start
    logger.info(
        f"Step 6/7 complete: Created Gradio components in {component_time:.2f}s"
    )

    logger.info("Step 7/7: Building Gradio interface and callbacks...")
    interface_start = time.time()
    with gr.Blocks(
        title="MTEB Leaderboard",
        fill_width=True,
    ) as demo:
        # Session tracking: Use State instead of BrowserState to avoid localStorage errors
        # JavaScript will manually handle localStorage when available
        session_data = gr.State("{}")
        session_id = gr.State("")

        # On page load: read from localStorage (if available) and initialize session
        demo.load(
            fn=on_page_load,
            inputs=[session_data],
            outputs=[session_data, session_id],
            js="""
            function(session_data) {
                const STORAGE_KEY = '__mteb_session_data__';
                let data = {};

                // Try to read from localStorage
                try {
                    const stored = localStorage.getItem(STORAGE_KEY);
                    if (stored) {
                        data = JSON.parse(stored);
                    }
                    data._storage_available = true;
                    window.__mteb_storage_available__ = true;
                } catch(e) {
                    console.warn('MTEB: localStorage unavailable, using fingerprint mode');
                    data._storage_available = false;
                    window.__mteb_storage_available__ = false;
                }

                return [JSON.stringify(data)];
            }
            """,
        )

        # After session is generated, save to localStorage if available
        session_data.change(
            fn=None,
            inputs=[session_data],
            js="""
            function(session_data) {
                if (window.__mteb_storage_available__) {
                    try {
                        localStorage.setItem('__mteb_session_data__', session_data);
                    } catch(e) {
                        console.warn('MTEB: Failed to save session to localStorage');
                    }
                }
                return session_data;
            }
            """,
        )

        with gr.Sidebar(
            position="left",
            label="Benchmark Selection and Customization",
            visible=True,
            width="18%",
        ):
            benchmark_select, column = _make_selector(
                GP_BENCHMARK_ENTRIES + R_BENCHMARK_ENTRIES
            )

        gr.Markdown(
            """
        ## Embedding Leaderboard

        This leaderboard compares 100+ text and image embedding models across 1000+ languages. We refer to the publication of each selectable benchmark for details on metrics, languages, tasks, and task types. Anyone is welcome [to add a model](https://embeddings-benchmark.github.io/mteb/contributing/adding_a_model/), [add benchmarks](https://embeddings-benchmark.github.io/mteb/contributing/adding_a_benchmark/), [help us improve zero-shot annotations](https://github.com/embeddings-benchmark/mteb/blob/06489abca007261c7e6b11f36d4844c5ed5efdcb/mteb/models/bge_models.py#L91) or [propose other changes to the leaderboard](https://github.com/embeddings-benchmark/mteb/issues/new?template=enhancement.yaml).
        """
        )
        gr.Markdown(
            lambda name: f"<center> <h2> <b> {name} </b> </h2> </center><br>",
            inputs=benchmark_select,
        )

        scores = gr.State(default_scores)
        models = gr.State(filtered_models)
        with gr.Row():
            with gr.Column(scale=1):
                description = gr.Markdown(
                    _update_description(
                        default_benchmark.name,
                        sorted(default_results.languages),
                        sorted(default_results.task_types),
                        sorted(default_results.domains),
                    )
                )

            with gr.Column(scale=1):
                with gr.Accordion("Cite and share this benchmark", open=False):
                    citation = gr.Markdown(_update_citation, inputs=[benchmark_select])  # noqa: F841
                    gr.Markdown(_produce_benchmark_link, inputs=[benchmark_select])

                with gr.Accordion(
                    "Customize this Benchmark",
                    open=False,
                ):
                    with gr.Column():
                        with gr.Row():
                            type_select.render()
                        with gr.Row():
                            domain_select.render()
                        with gr.Row():
                            modality_select.render()
                        with gr.Row(elem_classes="overflow-y-scroll max-h-80"):
                            lang_select.render()
                        with gr.Row(elem_classes="overflow-y-scroll max-h-80"):
                            task_select.render()

                with gr.Accordion("Advanced Model Filters", open=False):
                    with gr.Group():
                        with gr.Row(elem_classes=""):
                            with gr.Column():
                                compatibility = gr.CheckboxGroup(
                                    [
                                        (
                                            "Should be sentence-transformers compatible",
                                            "Sentence Transformers",
                                        )
                                    ],
                                    value=[],
                                    label="Compatibility",
                                    interactive=True,
                                )
                                availability = gr.Radio(
                                    [
                                        ("Only Open", True),
                                        ("Only Proprietary", False),
                                        ("Both", None),
                                    ],
                                    value=None,
                                    label="Availability",
                                    interactive=True,
                                )
                                instructions = gr.Radio(
                                    [
                                        ("Only Instruction-tuned", True),
                                        ("Only non-instruction", False),
                                        ("Both", None),
                                    ],
                                    value=None,
                                    label="Instructions",
                                    interactive=True,
                                )
                            with gr.Column():
                                zero_shot = gr.Radio(
                                    [
                                        (
                                            "Only Zero-shot",
                                            "only_zero_shot",
                                        ),
                                        ("Remove Unknown", "remove_unknown"),
                                        ("Allow All", "allow_all"),
                                    ],
                                    value="allow_all",
                                    label="Zero-shot",
                                    interactive=True,
                                )

                                max_model_size = gr.Radio(
                                    [
                                        ("<100M", 100),
                                        ("<500M", 500),
                                        ("<1B", 1000),
                                        ("<5B", 5000),
                                        ("<10B", 10000),
                                        (">10B", MAX_MODEL_SIZE),
                                    ],
                                    value=MAX_MODEL_SIZE,
                                    label="Model Parameters",
                                    interactive=True,
                                )
                            with gr.Column():
                                model_type_select = gr.CheckboxGroup(
                                    MODEL_TYPE_CHOICES,
                                    value=MODEL_TYPE_CHOICES,
                                    label="Model Type",
                                )

        with gr.Tab("Summary"):
            summary_table.render()
            download_summary = gr.DownloadButton("Download Table")
            download_summary.click(
                _download_table, inputs=[summary_table], outputs=[download_summary]
            )

            with gr.Accordion(
                "Frequently Asked Questions",
                open=False,
            ):
                gr.Markdown(FAQ)

        with gr.Tab("Performance per Model Size") as plot_tab:
            plot = gr.Plot(_performance_size_plot, inputs=[summary_table])
            plot_tab.select(
                _performance_size_plot, inputs=[summary_table], outputs=[plot]
            )

        with gr.Tab("Performance over Time") as timeline_tab:
            timeline_plot = gr.Plot(_performance_over_time_plot, inputs=[summary_table])
            timeline_tab.select(
                _performance_over_time_plot,
                inputs=[summary_table],
                outputs=[timeline_plot],
            )

        with gr.Tab(
            "Performance per Task Type", visible=display_radar_chart
        ) as radar_plot_tab:
            radar_plot = gr.Plot(_radar_chart, inputs=[summary_table])
            gr.Markdown(
                "*We only display TOP 5 models that have been run on all task types in the benchmark*"
            )
            radar_plot_tab.select(
                _radar_chart, inputs=[summary_table], outputs=[radar_plot]
            )

        with gr.Tab("Performance per task"):
            per_task_table.render()
            download_per_task = gr.DownloadButton("Download Table")
            download_per_task.click(
                _download_table, inputs=[per_task_table], outputs=[download_per_task]
            )
        with gr.Tab(
            "Performance per language", visible=display_language_table
        ) as language_tab:
            per_language_table.render()
            download_per_language = gr.DownloadButton("Download Table")
            download_per_language.click(
                _download_table,
                inputs=[per_language_table],
                outputs=[download_per_language],
            )
        with gr.Tab("Task information"):
            task_info_table = gr.DataFrame(
                _update_task_info(sorted(default_results.task_names)),
                datatype=["markdown"] + ["str"] * 6,
                buttons=["copy", "fullscreen"],
                show_search="filter",
            )

        # This sets the benchmark from the URL query parameters
        demo.load(_set_benchmark_on_load, inputs=[], outputs=[benchmark_select])

        def on_benchmark_select(benchmark_name, session_id=None):
            (
                languages,
                domains,
                types,
                modalities,
                benchmark_tasks,
                scores,
                show_zero_shot,
                initial_models,
            ) = _cache_on_benchmark_select(benchmark_name, all_benchmark_results)

            if session_id:
                event_logger.log_benchmark_change(
                    session_id=session_id, new_value=benchmark_name, old_value=None
                )

            benchmark_results = all_benchmark_results[benchmark_name]
            eligible_task_types = {
                task_type
                for task_type in benchmark_results.task_types
                if task_type != "InstructionRetrieval"
            }
            display_radar = len(eligible_task_types) > 1
            return (
                gr.update(choices=languages, value=languages),
                gr.update(choices=domains, value=domains),
                gr.update(choices=types, value=types),
                gr.update(choices=modalities, value=modalities),
                gr.update(choices=benchmark_tasks, value=benchmark_tasks),
                scores,
                gr.update(visible=show_zero_shot),
                initial_models,
                gr.update(visible=display_radar),
            )

        benchmark_select.change(
            on_benchmark_select,
            inputs=[benchmark_select, session_id],
            outputs=[
                lang_select,
                domain_select,
                type_select,
                modality_select,
                task_select,
                scores,
                zero_shot,
                models,
                radar_plot_tab,
            ],
        )
        for trigger in [lang_select, type_select, domain_select]:
            trigger.change(
                _update_description,
                inputs=[benchmark_select, lang_select, type_select, domain_select],
                outputs=[description],
                preprocess=False,
                show_progress="hidden",
            )
        task_select.change(
            _update_task_info,
            inputs=[task_select],
            outputs=[task_info_table],
            preprocess=False,
        )

        @cachetools.cached(
            cache={},
            key=lambda benchmark_name, languages: hash(
                (hash(benchmark_name), hash(tuple(languages)))
            ),
        )
        def update_scores_on_lang_change(benchmark_name, languages):
            start_time = time.time()
            if not len(languages):
                return []
            benchmark_results = all_benchmark_results[benchmark_name]
            scores = benchmark_results._get_scores(languages=languages, format="long")
            elapsed = time.time() - start_time
            logger.debug(f"update_scores callback: {elapsed}s")
            return scores

        lang_select.input(
            update_scores_on_lang_change,
            inputs=[benchmark_select, lang_select],
            outputs=[scores],
            preprocess=False,
        )

        def update_task_list(
            benchmark_name,
            type_select,
            domain_select,
            lang_select,
            modality_select,
            session_id=None,
        ):
            benchmark_tasks, tasks_to_keep = _cache_update_task_list(
                benchmark_name, type_select, domain_select, lang_select, modality_select
            )
            if session_id is not None:
                event_logger.log_filter_change(
                    session_id=session_id,
                    filter_name="task_type",
                    new_value=benchmark_name,
                    old_value=None,  # Optional: previous value
                    benchmark=benchmark_name,
                    filters={  # Optional: snapshot of all filters
                        "task_type": type_select,
                        "domain": domain_select,
                        "language": lang_select,
                        "modality": modality_select,
                    },
                )
            return gr.update(choices=benchmark_tasks, value=tasks_to_keep)

        type_select.input(
            update_task_list,
            inputs=[
                benchmark_select,
                type_select,
                domain_select,
                lang_select,
                modality_select,
                session_id,
            ],
            outputs=[task_select],
            preprocess=False,
        )
        domain_select.input(
            update_task_list,
            inputs=[
                benchmark_select,
                type_select,
                domain_select,
                lang_select,
                modality_select,
                session_id,
            ],
            outputs=[task_select],
            preprocess=False,
        )
        lang_select.input(
            update_task_list,
            inputs=[
                benchmark_select,
                type_select,
                domain_select,
                lang_select,
                modality_select,
                session_id,
            ],
            outputs=[task_select],
            preprocess=False,
        )
        modality_select.input(
            update_task_list,
            inputs=[
                benchmark_select,
                type_select,
                domain_select,
                lang_select,
                modality_select,
                session_id,
            ],
            outputs=[task_select],
            preprocess=False,
        )

        @cachetools.cached(
            cache={},
            key=lambda scores,
            tasks,
            availability,
            compatibility,
            instructions,
            max_model_size,
            zero_shot,
            model_type_select,
            session_id: hash(
                (
                    id(scores),
                    hash(tuple(tasks)),
                    hash(availability),
                    hash(tuple(compatibility)),
                    hash(instructions),
                    hash(max_model_size),
                    hash(zero_shot),
                    hash(tuple(model_type_select)),
                )
            ),
        )
        def update_models(
            scores: list[dict],
            tasks: list[str],
            availability: bool | None,
            compatibility: list[str],
            instructions: bool | None,
            max_model_size: int,
            zero_shot: Literal["allow_all", "remove_unknown", "only_zero_shot"],
            model_type_select: list[str],
            session_id,
        ):
            start_time = time.time()
            model_names = list({entry["model_name"] for entry in scores})
            filtered_models = _filter_models(
                model_names,
                tasks,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot_setting=zero_shot,
                model_types=model_type_select,
            )
            elapsed = time.time() - start_time
            logger.debug(f"update_models callback: {elapsed}s")
            # Always return sorted models to ensure models.change triggers update_tables
            if session_id is not None:
                event_logger.log_filter_change(
                    session_id=session_id,
                    filter_name="model_type",
                    new_value=None,
                    old_value=None,  # Optional: previous value
                    benchmark=None,
                    filters={  # Optional: snapshot of all filters
                        "scores": scores,
                        "tasks": tasks,
                        "availability": availability,
                        "compatibility": compatibility,
                        "instructions": instructions,
                        "max_model_size": max_model_size,
                        "zero_shot": zero_shot,
                    },
                )

            return sorted(filtered_models)

        scores.change(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )

        task_select.change(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )
        availability.input(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )
        compatibility.input(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )
        instructions.input(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )
        max_model_size.change(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )
        zero_shot.change(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )
        model_type_select.change(
            update_models,
            inputs=[
                scores,
                task_select,
                availability,
                compatibility,
                instructions,
                max_model_size,
                zero_shot,
                model_type_select,
                session_id,
            ],
            outputs=[models],
            preprocess=False,
        )

        def _cache_key_for_update_tables(
            scores, tasks, models_to_keep, benchmark_name, languages
        ):
            # Build a deterministic fingerprint from score content.
            # This keeps cache hits for equivalent data while invalidating on language-driven score changes.
            score_signature = []
            for entry in scores:
                score_signature.append(
                    (
                        entry.get("model_name"),
                        entry.get("task_name"),
                        entry.get("score"),
                    )
                )
            scores_hash = hash(tuple(sorted(score_signature)))
            tasks_hash = hash(tuple(sorted(tasks)))
            # Sort models_to_keep to ensure consistent hash regardless of input order
            models_hash = (
                hash(tuple(sorted(models_to_keep)))
                if models_to_keep is not None
                else None
            )
            bench_hash = hash(benchmark_name)
            lang_hash = (
                hash(tuple(sorted(languages))) if languages is not None else None
            )
            key = hash((scores_hash, tasks_hash, models_hash, bench_hash, lang_hash))

            return key

        @cachetools.cached(
            cache={},
            key=_cache_key_for_update_tables,
        )
        def update_tables(
            scores,
            tasks,
            models_to_keep,
            benchmark_name: str,
            languages: list[str],
        ):
            start_time = time.time()
            tasks = set(tasks)
            benchmark = mteb.get_benchmark(benchmark_name)

            # Extract filtered model and task names from scores (respects UI filters)
            filtered_model_names = set()
            filtered_task_names = set()

            for entry in scores:
                if entry["task_name"] not in tasks:
                    continue
                if (models_to_keep is not None) and (
                    entry["model_name"] not in models_to_keep
                ):
                    continue
                filtered_model_names.add(entry["model_name"])
                filtered_task_names.add(entry["task_name"])

            filtered_benchmark_results = _filter_benchmark_results_for_tables(
                benchmark_results=all_benchmark_results[benchmark_name],
                task_names=filtered_task_names,
                model_names=filtered_model_names,
                languages=languages,
            )

            summary = apply_summary_styling_from_benchmark(
                benchmark, filtered_benchmark_results
            )
            per_task = apply_per_task_styling_from_benchmark(
                benchmark, filtered_benchmark_results
            )
            per_language = apply_per_language_styling_from_benchmark(
                benchmark,
                filtered_benchmark_results,
            )
            elapsed = time.time() - start_time
            logger.debug(f"update_tables callback: {elapsed}s")
            return (
                summary,
                per_task,
                per_language,
                gr.update(visible=len(benchmark.language_view) > 0),
            )

        # Only update tables when models change, not when scores/tasks change directly
        # This avoids redundant updates since scores/tasks changes trigger update_models
        # which then triggers models.change
        for item in [models, task_select, lang_select]:
            item.change(
                update_tables,
                inputs=[scores, task_select, models, benchmark_select, lang_select],
                outputs=[
                    summary_table,
                    per_task_table,
                    per_language_table,
                    language_tab,
                ],
                preprocess=False,
            )

        gr.Markdown(ACKNOWLEDGEMENT, elem_id="ack_markdown")
    interface_time = time.time() - interface_start
    logger.info(f"Step 7/7 complete: Built Gradio interface in {interface_time:.2f}s")

    logger.info("Starting prerun on all benchmarks to populate caches...")
    prerun_start = time.time()
    # Prerun on all benchmarks, so that results of callbacks get cached
    for benchmark in benchmarks:
        (
            bench_languages,
            bench_domains,
            bench_types,
            bench_modalities,
            bench_tasks,
            bench_scores,
            zero_shot,
            bench_initial_models,
            display_radar,
        ) = on_benchmark_select(benchmark.name)
        # Call update_tables to populate cache (simulating models.change trigger)
        update_tables(
            bench_scores,
            bench_tasks,
            bench_initial_models,
            benchmark.name,
            bench_languages,
        )
        # Also cache the filtered tasks scenario
        filtered_tasks = update_task_list(
            benchmark.name,
            bench_types,
            bench_domains,
            bench_languages,
            bench_modalities,
        )
        update_tables(
            bench_scores,
            filtered_tasks,
            bench_initial_models,
            benchmark.name,
            bench_languages,
        )
    prerun_time = time.time() - prerun_start
    logger.info(
        f"Prerun complete: Processed {len(benchmarks)} benchmarks in {prerun_time:.2f}s"
    )

    total_time = time.time() - app_start
    logger.info(f"=== Leaderboard app initialization complete in {total_time:.2f}s ===")
    return demo


if __name__ == "__main__":
    import os

    # Add process ID to logging for multiprocessing debugging
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - PID:%(process)d - %(name)s - %(levelname)s - %(message)s",
        force=True,  # Override any existing handlers
    )

    # Flush log handlers immediately (helpful for multiprocessing)
    for handler in logging.root.handlers:
        handler.flush()

    logger.info(f"Starting leaderboard app in process {os.getpid()}")

    # Suppress specific WARNING messages while keeping INFO level for the app
    logging.getLogger("mteb.results.task_result").setLevel(logging.ERROR)
    logging.getLogger("mteb.models.model_meta").setLevel(logging.ERROR)
    logging.getLogger("mteb.results.benchmark_results").setLevel(logging.ERROR)

    warnings.filterwarnings("ignore", message="Couldn't get scores for .* due to .*")
    warnings.filterwarnings("ignore", message="Could not get source model: .*")
    warnings.filterwarnings(
        "ignore", message="No scores data available. Returning empty DataFrame."
    )
    warnings.filterwarnings("ignore", message="Main score .* not found in scores")
    warnings.filterwarnings("ignore", message=".*: Missing subsets .* for split .*")
    warnings.filterwarnings("ignore", message=".*: Missing splits .*")

    app = get_leaderboard_app()

    head = """
    <link href="https://cdn.jsdelivr.net/npm/tailwindcss@2.2.19/dist/tailwind.min.css" rel="stylesheet">
    """
    app.launch(
        server_name="0.0.0.0",
        server_port=7860,
        theme=gr.themes.Soft(
            font=[gr.themes.GoogleFont("Roboto Mono"), "Arial", "sans-serif"],
        ),
        head=head,
    )



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class PatchCamelyonClassification(AbsTaskClassification):
    input_column_name = "webp"
    label_column_name = "cls"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="PatchCamelyon",
        description="Histopathology diagnosis classification dataset.",
        reference="https://link.springer.com/chapter/10.1007/978-3-030-00934-2_24",
        dataset={
            "path": "mteb/wds_vtab-pcam",
            "revision": "6544d35f2235ed00e2ac8494c09ff107f8af353c",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2018-01-01",
            "2018-12-01",
        ),  # Estimated range for the collection of reviews
        domains=["Medical"],
        task_subtypes=["Tumor detection"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{10.1007/978-3-030-00934-2_24,
  address = {Cham},
  author = {Veeling, Bastiaan S.
and Linmans, Jasper
and Winkens, Jim
and Cohen, Taco
and Welling, Max},
  booktitle = {Medical Image Computing and Computer Assisted Intervention -- MICCAI 2018},
  editor = {Frangi, Alejandro F.
and Schnabel, Julia A.
and Davatzikos, Christos
and Alberola-L{\'o}pez, Carlos
and Fichtinger, Gabor},
  isbn = {978-3-030-00934-2},
  pages = {210--218},
  publisher = {Springer International Publishing},
  title = {Rotation Equivariant CNNs for Digital Pathology},
  year = {2018},
}
""",
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch

from mteb.models.model_meta import ModelMeta

from .rerankers_custom import RerankerWrapper

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import BatchedInput, PromptType

LISTCONRANKER_CITATION = """@article{liu2025listconranker,
  title={ListConRanker: A Contrastive Text Reranker with Listwise Encoding},
  author={Liu, Junlong and Ma, Yue and Zhao, Ruihui and Zheng, Junhao and Ma, Qianli and Kang, Yangyang},
  journal={arXiv preprint arXiv:2501.07111},
  year={2025}
}"""


class ListConRanker(RerankerWrapper):
    def __init__(self, model_name_or_path: str, **kwargs) -> None:
        from transformers import AutoModelForSequenceClassification, AutoTokenizer

        super().__init__(model_name_or_path, **kwargs)

        self.model = AutoModelForSequenceClassification.from_pretrained(
            model_name_or_path, trust_remote_code=True, torch_dtype=torch.float16
        )
        self.tokenizer = AutoTokenizer.from_pretrained(model_name_or_path)

        if not self.device:
            self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        model_args = {}
        if self.fp_options:
            model_args["torch_dtype"] = self.fp_options

        self.model = self.model.to(self.device)
        self.model.eval()

    @torch.inference_mode()
    def predict(
        self,
        inputs1: DataLoader[BatchedInput],
        inputs2: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ):
        queries = [text for batch in inputs1 for text in batch["query"]]
        passages = [text for batch in inputs2 for text in batch["text"]["text"]]

        assert len(queries) == len(passages)

        final_scores = []
        query = queries[0]
        tmp_passages = []
        if kwargs.get("traditional_inference"):
            for q, p in zip(queries, passages):
                if query == q:
                    tmp_passages.append(p)
                else:
                    query_passages_tuples = [[query] + tmp_passages]
                    scores = self.model.multi_passage(query_passages_tuples)
                    final_scores += scores
                    query = q
                    tmp_passages = [p]
            if len(tmp_passages) > 0:
                query_passages_tuples = [[query] + tmp_passages]
                scores = self.model.multi_passage(query_passages_tuples)
                final_scores += scores
        else:
            for q, p in zip(queries, passages):
                if query == q:
                    tmp_passages.append(p)
                else:
                    query_passages = [query] + tmp_passages
                    scores = self.model.multi_passage_in_iterative_inference(
                        query_passages
                    )
                    final_scores += scores
                    query = q
                    tmp_passages = [p]
            if len(tmp_passages) > 0:
                query_passages = [query] + tmp_passages
                scores = self.model.multi_passage_in_iterative_inference(query_passages)
                final_scores += scores

        assert len(final_scores) == len(queries), (
            f"Expected {len(queries)} scores, got {len(final_scores)}"
        )

        return final_scores


listconranker_training_datasets = {
    "CMedQAv1-reranking",
    "CMedQAv2-reranking",
    "MMarcoReranking",
    "T2Reranking",
    # 'Huatuo26M-Lite': ['train'],
    # 'MARC': ['train'],
    # 'XL-sum-chinese_simplified': ['train'],
    # 'CSL': ['train'],
}

listconranker = ModelMeta(
    loader=ListConRanker,
    loader_kwargs=dict(
        fp_options="float16",
    ),
    name="ByteDance/ListConRanker",
    model_type=["cross-encoder"],
    languages=["zho-Hans"],
    open_weights=True,
    revision="95ae6a5f422a916bc36520f0f3e198e7d91520a0",
    release_date="2024-12-11",
    n_parameters=401_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=1242,
    similarity_fn_name="cosine",
    training_datasets=listconranker_training_datasets,
    embed_dim=1024,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/ByteDance/ListConRanker",
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    citation=LISTCONRANKER_CITATION,
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, TypedDict

import torch
from datasets import Dataset
from sklearn import metrics

from mteb._evaluators import ZeroShotClassificationEvaluator
from mteb.models import EncoderProtocol
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

from ._statistics_calculation import (
    calculate_audio_statistics,
    calculate_image_statistics,
    calculate_label_statistics,
    calculate_text_statistics,
)
from .abstask import AbsTask

if TYPE_CHECKING:
    from pathlib import Path

    from mteb.models import MTEBModels
    from mteb.types import EncodeKwargs
    from mteb.types.statistics import (
        AudioStatistics,
        ImageStatistics,
        LabelStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


class ZeroShotClassificationDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for ZeroShotClassification

    Attributes:
        num_samples: number of samples in the dataset.
        number_of_characters: None (no text inputs)

        text_statistics: None (no text inputs)
        image_statistics: Statistics for images
        audio_statistics: None (no audio inputs)
        label_statistics: Statistics for dataset labels

        candidates_labels_text_statistics: Statistics for candidate labels text
    """

    num_samples: int
    number_of_characters: int | None

    text_statistics: TextStatistics | None
    image_statistics: ImageStatistics | None
    audio_statistics: AudioStatistics | None
    label_statistics: LabelStatistics
    candidates_labels_text_statistics: TextStatistics


class ZeroShotClassificationMetrics(TypedDict):
    """Metrics for ZeroShotClassification

    Attributes:
        accuracy: Accuracy of the model.
    """

    accuracy: float


class AbsTaskZeroShotClassification(AbsTask):
    """Abstract class for ZeroShot Classification tasks for any modality.

    The similarity between an input (can be image or text) and candidate text prompts, such as this is a dog/this is a cat.

    Attributes:
        dataset: Huggingface dataset containing the data for the task. Dataset must contain columns specified by self.input_column_name and self.label_column_name.
        input_column_name: Name of the column containing the inputs (image or text).
        label_column_name: Name of the column containing the labels (str).
    """

    input_column_name: str = "image"
    label_column_name: str = "label"

    def dataset_transform(self, num_proc: int | None = None, **kwargs: Any) -> None:
        """Keep only eval splits. Zero-shot tasks don't need train splits."""
        if self.dataset is None:
            return
        splits_to_keep = set(self.metadata.eval_splits)
        for split in list(self.dataset.keys()):
            if split not in splits_to_keep:
                del self.dataset[split]

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> ZeroShotClassificationDescriptiveStatistics:
        if hf_subset:
            inputs = self.dataset[hf_subset][split][self.input_column_name]
            labels = self.dataset[hf_subset][split][self.label_column_name]
        elif compute_overall:
            inputs, labels = [], []
            for hf_subset in self.metadata.eval_langs:
                inputs.extend(self.dataset[hf_subset][split][self.input_column_name])
                labels.extend(self.dataset[hf_subset][split][self.label_column_name])
        else:
            inputs = self.dataset[split][self.input_column_name]
            labels = self.dataset[split][self.label_column_name]

        num_samples = len(inputs)

        image_statistics = None
        text_statistics = None
        audio_statistics = None

        if "image" in self.metadata.modalities:
            image_statistics = calculate_image_statistics(inputs)
        if self.metadata.modalities == ["text"]:
            text_statistics = calculate_text_statistics(inputs)

        if "audio" in self.metadata.modalities:
            audio_statistics = calculate_audio_statistics(inputs)

        label_statistics = calculate_label_statistics(labels)
        candidate_lens = calculate_text_statistics(self.get_candidate_labels())

        return ZeroShotClassificationDescriptiveStatistics(
            num_samples=num_samples,
            number_of_characters=None,
            text_statistics=text_statistics,
            image_statistics=image_statistics,
            audio_statistics=audio_statistics,
            label_statistics=label_statistics,
            candidates_labels_text_statistics=candidate_lens,
        )

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: Dataset,
        *,
        hf_split: str,
        hf_subset: str,
        encode_kwargs: EncodeKwargs,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs,
    ) -> ZeroShotClassificationMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        candidate_labels = self.get_candidate_labels()
        data_split = data_split.select_columns(
            [self.input_column_name, self.label_column_name]
        )
        evaluator = ZeroShotClassificationEvaluator(
            data_split,
            self.input_column_name,
            candidate_labels,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **kwargs,
        )
        probs = evaluator(
            model,
            encode_kwargs=encode_kwargs,
            num_proc=num_proc,
        )

        if prediction_folder:
            self._save_task_predictions(
                probs.tolist(),
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        return self._calculate_scores(
            data_split[self.label_column_name],
            torch.tensor(probs).argmax(dim=1).tolist(),
        )

    def _calculate_scores(
        self,
        labels: list[int],
        predictions: list[float],
    ) -> ZeroShotClassificationMetrics:
        return ZeroShotClassificationMetrics(
            accuracy=metrics.accuracy_score(labels, predictions),
        )

    def _push_dataset_to_hub(
        self,
        repo_name: str,
        num_proc: int | None = None,
    ) -> None:
        self._upload_dataset_to_hub(
            repo_name,
            [
                self.input_column_name,
                self.label_column_name,
            ],
            num_proc=num_proc,
        )
        labels_dataset = Dataset.from_dict({"labels": self.get_candidate_labels()})
        labels_dataset.push_to_hub(repo_name, config_name="labels")

    def get_candidate_labels(self) -> list[str]:
        """Return the text candidates for zeroshot classification"""
        raise NotImplementedError("This method should be overridden by subclasses")



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SpanishPassageRetrievalS2S(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SpanishPassageRetrievalS2S",
        description="Test collection for passage retrieval from health-related Web resources in Spanish.",
        reference="https://mklab.iti.gr/results/spanish-passage-retrieval-dataset/",
        dataset={
            "path": "mteb/SpanishPassageRetrievalS2S",
            "revision": "6f276e4831066ee6d81d5f6d18b8949d17e65566",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["spa-Latn"],
        main_score="ndcg_at_10",
        date=("2019-01-01", "2019-12-31"),
        domains=["Medical", "Web", "Written"],
        task_subtypes=["Article retrieval"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{10.1007/978-3-030-15719-7_19,
  address = {Cham},
  author = {Kamateri, Eleni
and Tsikrika, Theodora
and Symeonidis, Spyridon
and Vrochidis, Stefanos
and Minker, Wolfgang
and Kompatsiaris, Yiannis},
  booktitle = {Advances in Information Retrieval},
  editor = {Azzopardi, Leif
and Stein, Benno
and Fuhr, Norbert
and Mayr, Philipp
and Hauff, Claudia
and Hiemstra, Djoerd},
  isbn = {978-3-030-15719-7},
  pages = {148--154},
  publisher = {Springer International Publishing},
  title = {A Test Collection for Passage Retrieval Evaluation of Spanish Health-Related Resources},
  year = {2019},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class LawIRKo(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="LawIRKo",
        description="""This dataset assesses a model's ability to retrieve relevant legal articles from queries referencing specific Korean laws and provisions. The corpus comprises official legal texts including statutes, acts, and regulations, with each document representing a single article. Queries are derived from law titles paired and article identifiers. For instance the law title might be "건축법" (Building Act) and the article name "기술적 기준" (Technical Standards), which would become "건축법에 명시된 법률 중에 '기술적 기준'에 대해 설명하고 있는 세부 항목은 무엇입니까?" ("Which specific articles in the Building Act explain the 'technical standards'?").""",
        reference="https://huggingface.co/datasets/on-and-on/lawgov_ir-ko",
        dataset={
            "path": "on-and-on/lawgov_ir-ko",
            "revision": "bd5361e486ef4be7052c506adfdf0610d04abbfe",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="ndcg_at_10",
        date=("2025-12-01", "2026-02-01"),
        domains=["Legal", "Written"],
        task_subtypes=["Question answering"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{law_ko_ir_khee,
  author = {kang-hyeun Lee},
  howpublished = {\url{https://huggingface.co/datasets/on-and-on/lawgov_ir-ko}},
  note = {A Benchmark Dataset for Korean Legal Information Retrieval and QA},
  year = {2026},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SwednRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SwednRetrieval",
        dataset={
            "path": "mteb/SwednRetrieval",
            "revision": "3920fa0e9ae78f6af3d8a20b33b39e1f9acdfdf9",
        },
        description="The SWE-DN corpus is based on 1,963,576 news articles from the Swedish newspaper Dagens Nyheter (DN) during the years 2000--2020. The articles are filtered to resemble the CNN/DailyMail dataset both regarding textual structure",
        reference="https://spraakbanken.gu.se/en/resources/swedn",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["swe-Latn"],
        main_score="ndcg_at_10",
        date=("2000-01-01", "2020-12-31"),
        domains=["News", "Non-fiction", "Written"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        task_subtypes=["Article retrieval"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{monsen2021method,
  author = {Monsen, Julius and J{\"o}nsson, Arne},
  booktitle = {Proceedings of CLARIN Annual Conference},
  title = {A method for building non-english corpora for abstractive text summarization},
  year = {2021},
}
""",
        prompt={
            "query": "Given a Swedish news headline retrieve summaries or news articles"
        },
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class GlobeV2GenderClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="GLOBEV2Gender",
        description="Gender classification from the GLOBE v2 dataset (sampled and enhanced from CommonVoice dataset for TTS purpose). This dataset is a stratified and downsampled version of the original dataset, containing about 535 hours of speech data across 164 accents. We use the gender column as the target label for audio classification.",
        reference="https://huggingface.co/datasets/MushanW/GLOBE_V2",
        dataset={
            "path": "mteb/globe-v2-gender-mini",
            "revision": "0907be05c04d6d811cc780e76b15eedd707601b2",
        },
        type="AudioClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2025-01-13", "2025-01-13"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Gender Classification"],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{wang2024globe,
  archiveprefix = {arXiv},
  author = {Wenbin Wang and Yang Song and Sanjay Jha},
  eprint = {2406.14875},
  title = {GLOBE: A High-quality English Corpus with Global Accents for Zero-shot Speaker Adaptive Text-to-Speech},
  year = {2024},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "gender"



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SWEbenchMultilingualRR(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SWEbenchMultilingualRR",
        description="Multilingual Software Issue Localization.",
        reference="https://www.swebench.com/multilingual.html",
        dataset={
            "path": "mteb/SWEbenchMultilingualRR",
            "revision": "2129d0fd2fe31e95b4eecbeca873b623d7f58c4d",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn", "python-Code"],
        main_score="recall_at_10",
        date=("2023-10-10", "2023-10-10"),
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        prompt={
            "query": "Given a github issue, identify the code that needs to be changed to fix the issue."
        },
        bibtex_citation=r"""
@misc{yang2025swesmith,
  archiveprefix = {arXiv},
  author = {John Yang and Kilian Lieret and Carlos E. Jimenez and Alexander Wettig and Kabir Khandpur and Yanzhe Zhang and Binyuan Hui and Ofir Press and Ludwig Schmidt and Diyi Yang},
  eprint = {2504.21798},
  primaryclass = {cs.SE},
  title = {SWE-smith: Scaling Data for Software Engineering Agents},
  url = {https://arxiv.org/abs/2504.21798},
  year = {2025},
}
""",
    )



from datasets import Dataset, DatasetDict

from mteb.abstasks.clustering import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata

_LANGS = {
    "ace_Latn": ["ace-Latn"],
    "acm_Arab": ["acm-Arab"],
    "acq_Arab": ["acq-Arab"],
    "aeb_Arab": ["aeb-Arab"],
    "afr_Latn": ["afr-Latn"],
    "ajp_Arab": ["ajp-Arab"],
    "aka_Latn": ["aka-Latn"],
    "als_Latn": ["als-Latn"],
    "amh_Ethi": ["amh-Ethi"],
    "apc_Arab": ["apc-Arab"],
    "arb_Latn": ["arb-Latn"],
    "ars_Arab": ["ars-Arab"],
    "ary_Arab": ["ary-Arab"],
    "arz_Arab": ["arz-Arab"],
    "asm_Beng": ["asm-Beng"],
    "ast_Latn": ["ast-Latn"],
    "awa_Deva": ["awa-Deva"],
    "ayr_Latn": ["ayr-Latn"],
    "azb_Arab": ["azb-Arab"],
    "azj_Latn": ["azj-Latn"],
    "bak_Cyrl": ["bak-Cyrl"],
    "bam_Latn": ["bam-Latn"],
    "ban_Latn": ["ban-Latn"],
    "bel_Cyrl": ["bel-Cyrl"],
    "bem_Latn": ["bem-Latn"],
    "ben_Beng": ["ben-Beng"],
    "bho_Deva": ["bho-Deva"],
    "bjn_Latn": ["bjn-Latn"],
    "bod_Tibt": ["bod-Tibt"],
    "bos_Latn": ["bos-Latn"],
    "bug_Latn": ["bug-Latn"],
    "bul_Cyrl": ["bul-Cyrl"],
    "cat_Latn": ["cat-Latn"],
    "ceb_Latn": ["ceb-Latn"],
    "ces_Latn": ["ces-Latn"],
    "cjk_Latn": ["cjk-Latn"],
    "ckb_Arab": ["ckb-Arab"],
    "crh_Latn": ["crh-Latn"],
    "cym_Latn": ["cym-Latn"],
    "dan_Latn": ["dan-Latn"],
    "deu_Latn": ["deu-Latn"],
    "dik_Latn": ["dik-Latn"],
    "dyu_Latn": ["dyu-Latn"],
    "dzo_Tibt": ["dzo-Tibt"],
    "ell_Grek": ["ell-Grek"],
    "eng_Latn": ["eng-Latn"],
    "epo_Latn": ["epo-Latn"],
    "est_Latn": ["est-Latn"],
    "eus_Latn": ["eus-Latn"],
    "ewe_Latn": ["ewe-Latn"],
    "fao_Latn": ["fao-Latn"],
    "fij_Latn": ["fij-Latn"],
    "fin_Latn": ["fin-Latn"],
    "fon_Latn": ["fon-Latn"],
    "fra_Latn": ["fra-Latn"],
    "fur_Latn": ["fur-Latn"],
    "fuv_Latn": ["fuv-Latn"],
    "gaz_Latn": ["gaz-Latn"],
    "gla_Latn": ["gla-Latn"],
    "gle_Latn": ["gle-Latn"],
    "glg_Latn": ["glg-Latn"],
    "grn_Latn": ["grn-Latn"],
    "guj_Gujr": ["guj-Gujr"],
    "hat_Latn": ["hat-Latn"],
    "hau_Latn": ["hau-Latn"],
    "heb_Hebr": ["heb-Hebr"],
    "hin_Deva": ["hin-Deva"],
    "hne_Deva": ["hne-Deva"],
    "hrv_Latn": ["hrv-Latn"],
    "hun_Latn": ["hun-Latn"],
    "hye_Armn": ["hye-Armn"],
    "ibo_Latn": ["ibo-Latn"],
    "ilo_Latn": ["ilo-Latn"],
    "ind_Latn": ["ind-Latn"],
    "isl_Latn": ["isl-Latn"],
    "ita_Latn": ["ita-Latn"],
    "jav_Latn": ["jav-Latn"],
    "jpn_Jpan": ["jpn-Jpan"],
    "kab_Latn": ["kab-Latn"],
    "kac_Latn": ["kac-Latn"],
    "kam_Latn": ["kam-Latn"],
    "kan_Knda": ["kan-Knda"],
    "kas_Deva": ["kas-Deva"],
    "kat_Geor": ["kat-Geor"],
    "kaz_Cyrl": ["kaz-Cyrl"],
    "kbp_Latn": ["kbp-Latn"],
    "kea_Latn": ["kea-Latn"],
    "khk_Cyrl": ["khk-Cyrl"],
    "khm_Khmr": ["khm-Khmr"],
    "kik_Latn": ["kik-Latn"],
    "kin_Latn": ["kin-Latn"],
    "kir_Cyrl": ["kir-Cyrl"],
    "kmb_Latn": ["kmb-Latn"],
    "kmr_Latn": ["kmr-Latn"],
    "knc_Latn": ["knc-Latn"],
    "kon_Latn": ["kon-Latn"],
    "kor_Hang": ["kor-Hang"],
    "lao_Laoo": ["lao-Laoo"],
    "lij_Latn": ["lij-Latn"],
    "lim_Latn": ["lim-Latn"],
    "lin_Latn": ["lin-Latn"],
    "lit_Latn": ["lit-Latn"],
    "lmo_Latn": ["lmo-Latn"],
    "ltg_Latn": ["ltg-Latn"],
    "ltz_Latn": ["ltz-Latn"],
    "lua_Latn": ["lua-Latn"],
    "lug_Latn": ["lug-Latn"],
    "luo_Latn": ["luo-Latn"],
    "lus_Latn": ["lus-Latn"],
    "lvs_Latn": ["lvs-Latn"],
    "mag_Deva": ["mag-Deva"],
    "mai_Deva": ["mai-Deva"],
    "mal_Mlym": ["mal-Mlym"],
    "mar_Deva": ["mar-Deva"],
    "min_Latn": ["min-Latn"],
    "mkd_Cyrl": ["mkd-Cyrl"],
    "mlt_Latn": ["mlt-Latn"],
    "mni_Beng": ["mni-Beng"],
    "mos_Latn": ["mos-Latn"],
    "mri_Latn": ["mri-Latn"],
    "mya_Mymr": ["mya-Mymr"],
    "nld_Latn": ["nld-Latn"],
    "nno_Latn": ["nno-Latn"],
    "nob_Latn": ["nob-Latn"],
    "npi_Deva": ["npi-Deva"],
    "nqo_Nkoo": ["nqo-Nkoo"],
    "nso_Latn": ["nso-Latn"],
    "nus_Latn": ["nus-Latn"],
    "nya_Latn": ["nya-Latn"],
    "oci_Latn": ["oci-Latn"],
    "ory_Orya": ["ory-Orya"],
    "pag_Latn": ["pag-Latn"],
    "pan_Guru": ["pan-Guru"],
    "pap_Latn": ["pap-Latn"],
    "pbt_Arab": ["pbt-Arab"],
    "pes_Arab": ["pes-Arab"],
    "plt_Latn": ["plt-Latn"],
    "pol_Latn": ["pol-Latn"],
    "por_Latn": ["por-Latn"],
    "prs_Arab": ["prs-Arab"],
    "quy_Latn": ["quy-Latn"],
    "ron_Latn": ["ron-Latn"],
    "run_Latn": ["run-Latn"],
    "rus_Cyrl": ["rus-Cyrl"],
    "sag_Latn": ["sag-Latn"],
    "san_Deva": ["san-Deva"],
    "sat_Olck": ["sat-Olck"],
    "scn_Latn": ["scn-Latn"],
    "shn_Mymr": ["shn-Mymr"],
    "sin_Sinh": ["sin-Sinh"],
    "slk_Latn": ["slk-Latn"],
    "slv_Latn": ["slv-Latn"],
    "smo_Latn": ["smo-Latn"],
    "sna_Latn": ["sna-Latn"],
    "snd_Arab": ["snd-Arab"],
    "som_Latn": ["som-Latn"],
    "sot_Latn": ["sot-Latn"],
    "spa_Latn": ["spa-Latn"],
    "srd_Latn": ["srd-Latn"],
    "srp_Cyrl": ["srp-Cyrl"],
    "ssw_Latn": ["ssw-Latn"],
    "sun_Latn": ["sun-Latn"],
    "swe_Latn": ["swe-Latn"],
    "swh_Latn": ["swh-Latn"],
    "szl_Latn": ["szl-Latn"],
    "tam_Taml": ["tam-Taml"],
    "taq_Tfng": ["taq-Tfng"],
    "tat_Cyrl": ["tat-Cyrl"],
    "tel_Telu": ["tel-Telu"],
    "tgk_Cyrl": ["tgk-Cyrl"],
    "tgl_Latn": ["tgl-Latn"],
    "tha_Thai": ["tha-Thai"],
    "tir_Ethi": ["tir-Ethi"],
    "tpi_Latn": ["tpi-Latn"],
    "tsn_Latn": ["tsn-Latn"],
    "tso_Latn": ["tso-Latn"],
    "tuk_Latn": ["tuk-Latn"],
    "tum_Latn": ["tum-Latn"],
    "tur_Latn": ["tur-Latn"],
    "twi_Latn": ["twi-Latn"],
    "tzm_Tfng": ["tzm-Tfng"],
    "uig_Arab": ["uig-Arab"],
    "ukr_Cyrl": ["ukr-Cyrl"],
    "umb_Latn": ["umb-Latn"],
    "urd_Arab": ["urd-Arab"],
    "uzn_Latn": ["uzn-Latn"],
    "vec_Latn": ["vec-Latn"],
    "vie_Latn": ["vie-Latn"],
    "war_Latn": ["war-Latn"],
    "wol_Latn": ["wol-Latn"],
    "xho_Latn": ["xho-Latn"],
    "ydd_Hebr": ["ydd-Hebr"],
    "yor_Latn": ["yor-Latn"],
    "yue_Hant": ["yue-Hant"],
    "zho_Hant": ["zho-Hant"],
    "zsm_Latn": ["zsm-Latn"],
    "zul_Latn": ["zul-Latn"],
}


class SIB200ClusteringFast(AbsTaskClustering):
    max_document_to_embed = 1004
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="SIB200ClusteringS2S",
        description="SIB-200 is the largest publicly available topic classification dataset based on Flores-200 covering 205 languages and dialects annotated. The dataset is annotated in English for the topics, science/technology, travel, politics, sports, health, entertainment, and geography. The labels are then transferred to the other languages in Flores-200 which are human-translated.",
        reference="https://arxiv.org/abs/2309.07445",
        dataset={
            "path": "mteb/sib200",
            "revision": "a74d7350ea12af010cfb1c21e34f1f81fd2e615b",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGS,
        main_score="v_measure",
        date=("2023-09-14", "2024-01-27"),
        domains=["News", "Written"],
        task_subtypes=["Thematic clustering"],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",  # expert annotated for English --> human translations
        dialect=[],
        sample_creation="human-translated and localized",
        bibtex_citation=r"""
@article{adelani2023sib,
  author = {Adelani, David Ifeoluwa and Liu, Hannah and Shen, Xiaoyu and Vassilyev, Nikita and Alabi, Jesujoba O and Mao, Yanke and Gao, Haonan and Lee, Annie En-Shiun},
  journal = {arXiv preprint arXiv:2309.07445},
  title = {SIB-200: A simple, inclusive, and big evaluation dataset for topic classification in 200+ languages and dialects},
  year = {2023},
}
""",  # combined train, validation, and test into test.
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        ds = {}
        for lang in self.hf_subsets:
            labels = []
            sentences = []
            ds[lang] = {}
            for split in ["train", "validation", "test"]:
                labels.extend(self.dataset[lang][split]["category"])
                sentences.extend(self.dataset[lang][split]["text"])
            split_ds = Dataset.from_dict({"labels": labels, "sentences": sentences})
            lang_dict = {"test": split_ds}
            ds[lang] = DatasetDict(lang_dict)
        self.dataset = DatasetDict(ds)
        for lang in self.hf_subsets:
            self.dataset[lang] = self.dataset[lang].class_encode_column("labels")



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class Flickr30kT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Flickr30kT2IRetrieval",
        description="Retrieve images based on captions.",
        reference="https://www.semanticscholar.org/paper/From-image-descriptions-to-visual-denotations%3A-New-Young-Lai/44040913380206991b1991daf1192942e038fe31",
        dataset={
            "path": "mteb/flickr30kt2i",
            "revision": "e819702b287bfbe084e129a61f308a802b7c108e",
        },
        type="Any2AnyRetrieval",
        category="t2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2018-01-01", "2018-12-31"),
        domains=["Web", "Written"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{Young2014FromID,
  author = {Peter Young and Alice Lai and Micah Hodosh and J. Hockenmaier},
  journal = {Transactions of the Association for Computational Linguistics},
  pages = {67-78},
  title = {From image descriptions to visual denotations: New similarity metrics for semantic inference over event descriptions},
  url = {https://api.semanticscholar.org/CorpusID:3104920},
  volume = {2},
  year = {2014},
}
""",
        prompt={"query": "Find an image that matches the given caption."},
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SlovakMovieReviewSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SlovakMovieReviewSentimentClassification",
        description="User reviews of movies on the CSFD movie database, with 2 sentiment classes (positive, negative)",
        reference="https://arxiv.org/pdf/2304.01922",
        dataset={
            "path": "mteb/SlovakMovieReviewSentimentClassification",
            "revision": "7d408f245f0d3df0eaaca08f89ab6a1936ada6f5",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["slk-Latn"],
        main_score="accuracy",
        date=("2002-05-21", "2020-03-05"),
        dialect=[],
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@misc{štefánik2023resources,
  archiveprefix = {arXiv},
  author = {Michal Štefánik and Marek Kadlčík and Piotr Gramacki and Petr Sojka},
  eprint = {2304.01922},
  primaryclass = {cs.CL},
  title = {Resources and Few-shot Learners for In-context Learning in Slavic Languages},
  year = {2023},
}
""",
        superseded_by="SlovakMovieReviewSentimentClassification.v2",
    )


class SlovakMovieReviewSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SlovakMovieReviewSentimentClassification.v2",
        description="User reviews of movies on the CSFD movie database, with 2 sentiment classes (positive, negative) This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://arxiv.org/pdf/2304.01922",
        dataset={
            "path": "mteb/slovak_movie_review_sentiment",
            "revision": "29a7405aabcfd4860a51ae6f65a5650d63108f26",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["slk-Latn"],
        main_score="accuracy",
        date=("2002-05-21", "2020-03-05"),
        dialect=[],
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        sample_creation="found",
        bibtex_citation=r"""
@misc{štefánik2023resources,
  archiveprefix = {arXiv},
  author = {Michal Štefánik and Marek Kadlčík and Piotr Gramacki and Petr Sojka},
  eprint = {2304.01922},
  primaryclass = {cs.CL},
  title = {Resources and Few-shot Learners for In-context Learning in Slavic Languages},
  year = {2023},
}
""",
        adapted_from=["SlovakMovieReviewSentimentClassification"],
    )

    def dataset_transform(self, num_proc: int | None = None, **kwargs) -> None:
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )



from sklearn.linear_model import LogisticRegression
from sklearn.multioutput import MultiOutputClassifier

from mteb.abstasks import AbsTaskMultilabelClassification
from mteb.abstasks.task_metadata import TaskMetadata


class AudioSetMultilingualClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="AudioSet",
        description="AudioSet consists of an expanding ontology of 632 audio event classes and a collection of 2,084,320 human-labeled 10-second sound clips drawn from YouTube videos.",
        reference="https://huggingface.co/datasets/agkphysics/AudioSet",
        dataset={
            "path": "agkphysics/AudioSet",
            "revision": "0c609e8302cf139307f639c57652032af0a88041",
        },
        type="AudioMultilabelClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="lrap",
        date=(
            "2016-01-01",
            "2017-01-30",
        ),
        domains=["Web", "Music", "Speech", "Scene"],
        task_subtypes=[
            "Environment Sound Classification",
            "Music Instrument Recognition",
            "Vocal Sound Classification",
            "Gunshot Audio Classification",
        ],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{audioset,
  author = {Gemmeke, Jort F and Ellis, Daniel PW and Freedman, Dylan and Jansen, Aren and Lawrence, Wade and Moore, R Channing and Plakal, Manoj and Ritter, Marvin},
  booktitle = {2017 IEEE international conference on acoustics, speech and signal processing (ICASSP)},
  organization = {IEEE},
  pages = {776--780},
  title = {Audio set: An ontology and human-labeled dataset for audio events},
  year = {2017},
}
""",
        superseded_by="AudioSetMini",
    )

    evaluator_model = MultiOutputClassifier(estimator=LogisticRegression())
    input_column_name: str = "audio"
    label_column_name: str = "human_labels"


# Sampled using scripts/data/audioset/create_data.ipynb
class AudioSetMiniMultilingualClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="AudioSetMini",
        description="AudioSet consists of an expanding ontology of 632 audio event classes and a collection of 2,084,320 human-labeled 10-second sound clips drawn from YouTube videos. This is a mini version that is sampled from the original dataset.",
        reference="https://huggingface.co/datasets/agkphysics/AudioSet",
        dataset={
            "path": "mteb/audioset",
            "revision": "168a7e681ee40609129535d49855c7e3e77e5efa",
        },
        type="AudioMultilabelClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="lrap",
        date=(
            "2016-01-01",
            "2017-01-30",
        ),
        domains=["Web", "Music", "Speech", "Scene"],
        task_subtypes=[
            "Environment Sound Classification",
            "Music Instrument Recognition",
            "Vocal Sound Classification",
            "Gunshot Audio Classification",
        ],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{audioset,
  author = {Gemmeke, Jort F and Ellis, Daniel PW and Freedman, Dylan and Jansen, Aren and Lawrence, Wade and Moore, R Channing and Plakal, Manoj and Ritter, Marvin},
  booktitle = {2017 IEEE international conference on acoustics, speech and signal processing (ICASSP)},
  organization = {IEEE},
  pages = {776--780},
  title = {Audio set: An ontology and human-labeled dataset for audio events},
  year = {2017},
}
""",
    )

    evaluator_model = MultiOutputClassifier(estimator=LogisticRegression())
    input_column_name: str = "audio"
    label_column_name: str = "human_labels"



from .afri_senti_classification import AfriSentiClassification
from .afri_senti_lang_classification import AfriSentiLangClassification
from .amazon_counterfactual_classification import AmazonCounterfactualClassification
from .amazon_reviews_classification import AmazonReviewsClassification
from .catalonia_tweet_classification import CataloniaTweetClassification
from .cyrillic_turkic_lang_classification import CyrillicTurkicLangClassification
from .hin_dialect_classification import HinDialectClassification
from .hume_multilingual_sentiment_classification import (
    HUMEMultilingualSentimentClassification,
)
from .indic_lang_classification import IndicLangClassification
from .indic_nlp_news_classification import IndicNLPNewsClassification
from .indic_sentiment_classification import IndicSentimentClassification
from .language_classification import LanguageClassification
from .m_in_ds14 import MInDS14Classification
from .masakha_news_classification import MasakhaNEWSClassification
from .massive_intent_classification import MassiveIntentClassification
from .massive_scenario_classification import MassiveScenarioClassification
from .mtop_domain_classification import MTOPDomainClassification
from .mtop_intent_classification import MTOPIntentClassification
from .multi_hate_classification import MultiHateClassification
from .multilingual_sentiment_classification import MultilingualSentimentClassification
from .naija_senti import NaijaSenti
from .nordic_lang_classification import NordicLangClassification
from .nusa_paragraph_emotion_classification import NusaParagraphEmotionClassification
from .nusa_paragraph_topic_classification import NusaParagraphTopicClassification
from .nusa_x_senti import NusaXSentiClassification
from .ru_nlu_intent_classification import RuNLUIntentClassification
from .ru_sci_bench_classification import (
    RuSciBenchCoreRiscClassification,
    RuSciBenchGRNTIClassificationV2,
    RuSciBenchOECDClassificationV2,
    RuSciBenchPubTypeClassification,
)
from .scala_classification import ScalaClassification
from .scandi_sent_classification import ScandiSentClassification
from .sib200_classification import SIB200Classification
from .sibfleurs import SIBFLEURSMultilingualClassification
from .south_african_lang_classification import SouthAfricanLangClassification
from .swiss_judgement_classification import SwissJudgementClassification
from .turkic_classification import TurkicClassification
from .tweet_sentiment_classification import TweetSentimentClassification
from .vox_populi_gender_id import VoxPopuliGenderID
from .vox_populi_language_id import VoxPopuliLanguageID

__all__ = [
    "AfriSentiClassification",
    "AfriSentiLangClassification",
    "AmazonCounterfactualClassification",
    "AmazonReviewsClassification",
    "CataloniaTweetClassification",
    "CyrillicTurkicLangClassification",
    "HUMEMultilingualSentimentClassification",
    "HinDialectClassification",
    "IndicLangClassification",
    "IndicNLPNewsClassification",
    "IndicSentimentClassification",
    "LanguageClassification",
    "MInDS14Classification",
    "MTOPDomainClassification",
    "MTOPIntentClassification",
    "MasakhaNEWSClassification",
    "MassiveIntentClassification",
    "MassiveScenarioClassification",
    "MultiHateClassification",
    "MultilingualSentimentClassification",
    "NaijaSenti",
    "NordicLangClassification",
    "NusaParagraphEmotionClassification",
    "NusaParagraphTopicClassification",
    "NusaXSentiClassification",
    "RuNLUIntentClassification",
    "RuSciBenchCoreRiscClassification",
    "RuSciBenchGRNTIClassificationV2",
    "RuSciBenchOECDClassificationV2",
    "RuSciBenchPubTypeClassification",
    "SIB200Classification",
    "SIBFLEURSMultilingualClassification",
    "ScalaClassification",
    "ScandiSentClassification",
    "SouthAfricanLangClassification",
    "SwissJudgementClassification",
    "TurkicClassification",
    "TweetSentimentClassification",
    "VoxPopuliGenderID",
    "VoxPopuliLanguageID",
]



from .sinhala_news_classification import (
    SinhalaNewsClassification,
    SinhalaNewsClassificationV2,
)
from .sinhala_news_source_classification import (
    SinhalaNewsSourceClassification,
    SinhalaNewsSourceClassificationV2,
)

__all__ = [
    "SinhalaNewsClassification",
    "SinhalaNewsClassificationV2",
    "SinhalaNewsSourceClassification",
    "SinhalaNewsSourceClassificationV2",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_ria_news_metadata = dict(
    reference="https://arxiv.org/abs/1901.07786",
    type="Retrieval",
    category="t2t",
    modalities=["text"],
    eval_splits=["test"],
    eval_langs=["rus-Cyrl"],
    main_score="ndcg_at_10",
    date=("2010-01-01", "2014-12-31"),
    domains=["News", "Written"],
    task_subtypes=["Article retrieval"],
    license="cc-by-nc-nd-4.0",
    annotations_creators="derived",
    dialect=[],
    sample_creation="found",
    bibtex_citation=r"""
@inproceedings{gavrilov2018self,
  author = {Gavrilov, Daniil and  Kalaidin, Pavel and  Malykh, Valentin},
  booktitle = {Proceedings of the 41st European Conference on Information Retrieval},
  title = {Self-Attentive Model for Headline Generation},
  year = {2019},
}
""",
)


class RiaNewsRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="RiaNewsRetrieval",
        dataset={
            "path": "ai-forever/ria-news-retrieval",
            "revision": "82374b0bbacda6114f39ff9c5b925fa1512ca5d7",
        },
        description="News article retrieval by headline. Based on Rossiya Segodnya dataset.",
        prompt={"query": "Given a news title, retrieve relevant news article"},
        **_ria_news_metadata,
    )


class RiaNewsRetrievalHardNegatives(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="RiaNewsRetrievalHardNegatives",
        dataset={
            "path": "mteb/RiaNewsRetrieval_test_top_250_only_w_correct-v2",
            "revision": "d42860a6c15f0a2c4485bda10c6e5b641fdfe479",
        },
        description="News article retrieval by headline. Based on Rossiya Segodnya dataset. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        adapted_from=["RiaNewsRetrieval"],
        superseded_by="RiaNewsRetrievalHardNegatives.v2",
        **_ria_news_metadata,
    )


class RiaNewsRetrievalHardNegativesV2(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="RiaNewsRetrievalHardNegatives.v2",
        dataset={
            "path": "mteb/RiaNewsRetrieval_test_top_250_only_w_correct-v2",
            "revision": "d42860a6c15f0a2c4485bda10c6e5b641fdfe479",
        },
        description=(
            "News article retrieval by headline. Based on Rossiya Segodnya dataset. "
            "The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct."
            "V2 uses a more appropriate prompt rather than the default prompt for retrieval. You can get more information on the effect of different prompt in the [PR](https://github.com/embeddings-benchmark/mteb/pull/3469#issuecomment-3436467106)"
        ),
        adapted_from=["RiaNewsRetrieval"],
        prompt={"query": "Given a news title, retrieve relevant news article"},
        **_ria_news_metadata,
    )



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

import numpy as np

from mteb._create_dataloaders import _create_text_queries_dataloader
from mteb._requires_package import requires_package
from mteb.models.model_meta import ModelMeta

if TYPE_CHECKING:
    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.models.models_protocols import SearchProtocol
    from mteb.types import (
        CorpusDatasetType,
        EncodeKwargs,
        QueryDatasetType,
        RetrievalOutputType,
        TopRankedDocumentsType,
    )

logger = logging.getLogger(__name__)


def _composite_prior(
    query_tfs: np.ndarray,
    doc_lengths: np.ndarray,
    avg_dl: float,
) -> np.ndarray:
    """Composite prior from Bayesian BM25 paper Section 4.2.

    Combines term frequency prior (Def 4.2.1) and field norm prior
    (Def 4.2.2) to produce document-level prior probabilities.

    Args:
        query_tfs: Total query term frequency per candidate document.
        doc_lengths: Token count per candidate document.
        avg_dl: Average document length across the corpus.

    Returns:
        Prior probabilities clipped to [0.1, 0.9].
    """
    p_tf = 0.2 + 0.7 * np.minimum(1.0, query_tfs / 10.0)

    if avg_dl > 0:
        norm_ratio = doc_lengths / avg_dl
    else:
        norm_ratio = np.ones_like(doc_lengths, dtype=np.float64)
    p_norm = 0.3 + 0.6 * (1.0 - np.minimum(1.0, np.abs(norm_ratio - 1.0) * 0.5))

    prior = 0.7 * p_tf + 0.3 * p_norm
    return np.clip(prior, 0.1, 0.9)


def bb25_loader(model_name, **kwargs) -> SearchProtocol:
    requires_package(bb25_loader, "bm25s", model_name, "pip install mteb[bm25s]")
    import bm25s
    import Stemmer

    class BB25Search:
        """Bayesian BM25 search using bm25s as the BM25 backend.

        Bayesian BM25 transforms traditional BM25 scores into calibrated
        probability estimates in [0, 1] through Bayesian inference with a
        sigmoid likelihood model and composite prior design.

        With the default prior_weight=0.0, the prior is a flat 0.5 for
        all documents. Since the sigmoid likelihood is strictly monotonic,
        this preserves BM25 rankings exactly while outputting calibrated
        probabilities suitable for hybrid search score fusion.

        Setting prior_weight > 0 enables the Composite Prior (Section 4.2
        of the paper), which re-adjusts rankings based on document-level
        evidence from query term frequency and document length signals.

        Architecture:
            1. bm25s handles fast BM25 indexing and top-k retrieval.
            2. Retrieved candidates are re-scored with Bayesian posterior:
               - Dynamic beta = median(BM25 scores) per query
               - Sigmoid likelihood: sigma(alpha * (score - beta))
               - prior = 0.5 + prior_weight * (composite_prior - 0.5)
               - Posterior via Bayes' rule
        """

        retriever: bm25s.BM25
        corpus_idx_to_id: dict[int, str]

        def __init__(
            self,
            previous_results: str | None = None,
            stopwords: str = "en",
            stemmer_language: str | None = "english",
            k1: float = 1.5,
            b: float = 0.75,
            alpha: float = 1.0,
            prior_weight: float = 0.0,
            **kwargs,
        ):
            self.k1 = k1
            self.b = b
            self.alpha = alpha
            self.prior_weight = prior_weight
            self.stopwords = stopwords
            self.stemmer = (
                Stemmer.Stemmer(stemmer_language) if stemmer_language else None
            )

        def _encode(self, texts: list[str]):
            """Tokenize texts using bm25s. Not to be confused with EncoderProtocol.encode()."""
            return bm25s.tokenize(texts, stopwords=self.stopwords, stemmer=self.stemmer)

        def index(
            self,
            corpus: CorpusDatasetType,
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            encode_kwargs: EncodeKwargs,
            num_proc: int | None = None,
        ) -> None:
            logger.info("Encoding Corpus...")
            corpus_texts = [
                "\n".join([doc.get("title", ""), doc["text"]]) for doc in corpus
            ]
            encoded_corpus = self._encode(corpus_texts)

            logger.info(
                f"Indexing Corpus... {len(encoded_corpus.ids):,} documents, "
                f"{len(encoded_corpus.vocab):,} vocab"
            )

            self.retriever = bm25s.BM25(k1=self.k1, b=self.b)
            self.retriever.index(encoded_corpus)
            self.corpus_idx_to_id = {i: row["id"] for i, row in enumerate(corpus)}

            if self.prior_weight > 0:
                # Per-document token IDs for composite prior computation.
                # Stored as compact numpy int32 arrays to minimize memory.
                self.corpus_token_ids = [
                    np.array(doc_ids, dtype=np.int32) for doc_ids in encoded_corpus.ids
                ]
                self.corpus_vocab = dict(encoded_corpus.vocab)
                self.doc_lengths = np.array(
                    [len(ids) for ids in encoded_corpus.ids], dtype=np.float64
                )
                self.avg_dl = (
                    float(self.doc_lengths.mean()) if len(self.doc_lengths) > 0 else 0.0
                )

            logger.info(f"Indexed {len(self.corpus_idx_to_id):,} documents")

        def search(
            self,
            queries: QueryDatasetType,
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            top_k: int,
            encode_kwargs: EncodeKwargs,
            top_ranked: TopRankedDocumentsType | None = None,
            num_proc: int | None = None,
        ) -> RetrievalOutputType:
            logger.info("Encoding Queries...")
            query_ids = list(queries["id"])
            results: RetrievalOutputType = {qid: {} for qid in query_ids}
            queries_loader = _create_text_queries_dataloader(queries)
            queries_texts = [text for batch in queries_loader for text in batch["text"]]

            query_tokenized = self._encode(queries_texts)

            logger.info(f"Retrieving Results... {len(queries):,} queries")

            queries_results, queries_scores = self.retriever.retrieve(
                query_tokenized,
                k=min(top_k, len(self.corpus_idx_to_id)),
            )

            use_prior = self.prior_weight > 0
            if use_prior:
                query_id_to_str = {v: k for k, v in query_tokenized.vocab.items()}

            for qi, qid in enumerate(query_ids):
                doc_indices = queries_results[qi]
                bm25_scores = queries_scores[qi].astype(np.float64)

                query_documents = (
                    top_ranked[qid] if top_ranked and qid in top_ranked else None
                )

                doc_id_to_score: dict[str, float] = {}

                # Separate positive-score candidates for Bayesian re-scoring
                positive_mask = bm25_scores > 0
                positive_indices = np.where(positive_mask)[0]

                if len(positive_indices) > 0:
                    cand_doc_indices = doc_indices[positive_indices]
                    cand_bm25_scores = bm25_scores[positive_indices]

                    # Dynamic beta = median of BM25 scores for this query
                    beta = float(np.median(cand_bm25_scores))

                    # Dynamic alpha scaling for query-level score distribution
                    # invariance. The paper defines alpha as sigmoid steepness;
                    # dividing by std(scores) keeps the effective steepness
                    # consistent across queries whose BM25 ranges vary widely,
                    # preventing sigmoid saturation on high-scoring queries.
                    score_std = float(np.std(cand_bm25_scores))
                    alpha_eff = (
                        self.alpha / score_std if score_std > 1e-10 else self.alpha
                    )

                    # Sigmoid likelihood (monotonic -- preserves BM25 ranking)
                    x = np.clip(alpha_eff * (cand_bm25_scores - beta), -500, 500)
                    likelihood = 1.0 / (1.0 + np.exp(-x))

                    if use_prior:
                        # Map query token IDs to corpus vocab IDs
                        query_term_strs = [
                            query_id_to_str[tid] for tid in query_tokenized.ids[qi]
                        ]
                        query_corpus_ids = np.array(
                            [
                                self.corpus_vocab[t]
                                for t in query_term_strs
                                if t in self.corpus_vocab
                            ],
                            dtype=np.int32,
                        )

                        n_cand = len(cand_doc_indices)
                        query_tfs = np.zeros(n_cand, dtype=np.float64)
                        cand_doc_lengths = np.zeros(n_cand, dtype=np.float64)

                        has_query_terms = len(query_corpus_ids) > 0
                        for ci in range(n_cand):
                            doc_idx = cand_doc_indices[ci]
                            cand_doc_lengths[ci] = self.doc_lengths[doc_idx]
                            if has_query_terms:
                                doc_ids = self.corpus_token_ids[doc_idx]
                                query_tfs[ci] = np.isin(doc_ids, query_corpus_ids).sum()

                        composite = _composite_prior(
                            query_tfs, cand_doc_lengths, self.avg_dl
                        )
                        # Interpolate between flat prior (0.5) and composite
                        prior = 0.5 + self.prior_weight * (composite - 0.5)
                        posterior = (likelihood * prior) / (
                            likelihood * prior + (1.0 - likelihood) * (1.0 - prior)
                        )
                    else:
                        # prior=0.5 => posterior = likelihood
                        posterior = likelihood

                    for ci in range(len(cand_doc_indices)):
                        doc_id = self.corpus_idx_to_id[cand_doc_indices[ci]]
                        if query_documents is None or doc_id in query_documents:
                            doc_id_to_score[doc_id] = float(posterior[ci])

                # Include zero-score documents with score 0.0
                for vi in np.where(~positive_mask)[0]:
                    doc_id = self.corpus_idx_to_id[doc_indices[vi]]
                    if query_documents is None or doc_id in query_documents:
                        doc_id_to_score[doc_id] = 0.0

                results[qid] = doc_id_to_score

            return results

    return BB25Search(**kwargs)


bb25_model = ModelMeta(
    loader=bb25_loader,
    name="mteb/baseline-bb25",
    model_type=["sparse"],
    languages=None,
    open_weights=True,
    revision="0_1_1",
    release_date="2026-02-06",
    n_parameters=None,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    embed_dim=None,
    license=None,
    max_tokens=None,
    reference="https://github.com/instructkr/bb25",
    similarity_fn_name=None,
    framework=[],
    use_instructions=False,
    public_training_code="https://github.com/instructkr/bb25",
    public_training_data=None,
    training_datasets=None,
    citation="""@software{jeong2026bayesianbm25,
  title={Bayesian BM25: A Probabilistic Framework for Hybrid Text and Vector Search},
  author={Jeong, Jaepil},
  year={2026},
  doi={10.5281/zenodo.18414941},
  url={https://doi.org/10.5281/zenodo.18414941},
}
@software{jeong2026neural,
  title={From Bayesian Inference to Neural Computation: The Analytical Emergence of Neural Network Structure from Probabilistic Relevance Estimation},
  author={Jeong, Jaepil},
  year={2026},
  doi={10.5281/zenodo.18512411},
  url={https://doi.org/10.5281/zenodo.18512411},
}""",
)



import datasets

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SNLRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SNLRetrieval",
        dataset={
            "path": "adrlau/navjordj-SNL_summarization_copy",  # TODO: replace with mteb/SNLRetrieval after #2820 is resolved.
            "revision": "22c474c88fb4678052f9099bb917ad8f9e155f9f",
        },
        description="Webscrabed articles and ingresses from the Norwegian lexicon 'Det Store Norske Leksikon'.",
        reference="https://huggingface.co/datasets/mteb/SNLRetrieval",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nob-Latn"],
        main_score="ndcg_at_10",
        date=("2020-01-01", "2024-12-31"),  # best guess
        domains=["Encyclopaedic", "Non-fiction", "Written"],
        license="cc-by-nc-4.0",  # version assumed (not specified beforehand)
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@mastersthesis{navjord2023beyond,
  author = {Navjord, J{\o}rgen Johnsen and Korsvik, Jon-Mikkel Ryen},
  school = {Norwegian University of Life Sciences, {\AA}s},
  title = {Beyond extractive: advancing abstractive automatic text summarization in Norwegian with transformers},
  year = {2023},
}
""",
        prompt={"query": "Given a lexicon headline in Norwegian, retrieve its article"},
        task_subtypes=["Article retrieval"],
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(**self.metadata.dataset)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, num_proc: int | None = None, **kwargs) -> None:
        """And transform to a retrieval dataset, which have the following attributes

        self.corpus = dict[doc_id, dict[str, str]] #id => dict with document data like title and text
        self.queries = dict[query_id, str] #id => query
        self.relevant_docs = dict[query_id, dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            ds: datasets.Dataset = self.dataset[split]
            ds = ds.shuffle(seed=42)

            self.queries[split] = {}
            self.relevant_docs[split] = {}
            self.corpus[split] = {}

            headline = ds["headline"]
            article = ds["article"]

            n = 0
            for headl, art in zip(headline, article):
                self.queries[split][str(n)] = headl
                q_n = n
                n += 1
                if art not in text2id:
                    text2id[art] = n
                    self.corpus[split][str(n)] = {"title": "", "text": art}
                    n += 1
                self.relevant_docs[split][str(q_n)] = {
                    str(text2id[art]): 1
                }  # only one correct matches



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class MalayalamNewsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MalayalamNewsClassification",
        description="A Malayalam dataset for 3-class classification of Malayalam news articles",
        reference="https://github.com/goru001/nlp-for-malyalam",
        dataset={
            "path": "mlexplorer008/malayalam_news_classification",
            "revision": "666f63bba2387456d8f846ea4d0565181bd47b81",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2014-01-01", "2018-01-01"),
        eval_splits=["test"],
        eval_langs=["mal-Mlym"],
        main_score="accuracy",
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{kunchukuttan2020indicnlpcorpus,
  author = {Anoop Kunchukuttan and Divyanshu Kakwani and Satish Golla and Gokul N.C. and Avik Bhattacharyya and Mitesh M. Khapra and Pratyush Kumar},
  journal = {arXiv preprint arXiv:2005.00085},
  title = {AI4Bharat-IndicNLP Corpus: Monolingual Corpora and Word Embeddings for Indic Languages},
  year = {2020},
}
""",
        superseded_by="MalayalamNewsClassification.v2",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        self.dataset = self.dataset.rename_columns({"headings": "text"})


class MalayalamNewsClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MalayalamNewsClassification.v2",
        description="A Malayalam dataset for 3-class classification of Malayalam news articles This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://github.com/goru001/nlp-for-malyalam",
        dataset={
            "path": "mteb/malayalam_news",
            "revision": "2bb37780ab4a68cb0b28a902059463563b2dbab9",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2014-01-01", "2018-01-01"),
        eval_splits=["test"],
        eval_langs=["mal-Mlym"],
        main_score="accuracy",
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{kunchukuttan2020indicnlpcorpus,
  author = {Anoop Kunchukuttan and Divyanshu Kakwani and Satish Golla and Gokul N.C. and Avik Bhattacharyya and Mitesh M. Khapra and Pratyush Kumar},
  journal = {arXiv preprint arXiv:2005.00085},
  title = {AI4Bharat-IndicNLP Corpus: Monolingual Corpora and Word Embeddings for Indic Languages},
  year = {2020},
}
""",
        adapted_from=["MalayalamNewsClassification"],
    )



from mteb.models.model_meta import (
    ModelMeta,
    ScoringFunction,
)
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader
from mteb.types import PromptType

from .facebookai import XLMR_LANGUAGES

E5_PAPER_RELEASE_DATE = "2024-02-08"


MULTILINGUAL_E5_CITATION = """
@article{wang2024multilingual,
  title={Multilingual E5 Text Embeddings: A Technical Report},
  author={Wang, Liang and Yang, Nan and Huang, Xiaolong and Yang, Linjun and Majumder, Rangan and Wei, Furu},
  journal={arXiv preprint arXiv:2402.05672},
  year={2024}
}
"""

E5_CITATION = """
@article{wang2022text,
  title={Text Embeddings by Weakly-Supervised Contrastive Pre-training},
  author={Wang, Liang and Yang, Nan and Huang, Xiaolong and Jiao, Binxing and Yang, Linjun and Jiang, Daxin and Majumder, Rangan and Wei, Furu},
  journal={arXiv preprint arXiv:2212.03533},
  year={2022}
}
"""

model_prompts = {
    PromptType.query.value: "query: ",
    PromptType.document.value: "passage: ",
}

E5_TRAINING_DATA = {
    # from 4.2 in https://arxiv.org/pdf/2212.03533
    # also pre-training data from a variety of sources (stackexchange, semantic scholar, reddit, CC, ...)
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "MSMARCO-PL",  # translation not trained on
    "mMARCO-NL",  # translation not trained on
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",  # translation not trained on
    "NQ-NL",  # translation not trained on
}

ME5_TRAINING_DATA = {
    "XQuADRetrieval",  # trained on SQuAD train dataset
    "FEVER",
    "FEVERHardNegatives",
    "FEVER-NL",  # translation not trained on
    "FEVER-PL",  # translation not trained on
    "HotpotQA",
    "HotpotQAHardNegatives",
    "HotpotQA-PL",  # translation not trained on
    "HotpotQA-NL",  # translation not trained on
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLReranking",
    "MrTidyRetrieval",
} | E5_TRAINING_DATA

e5_mult_small = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/multilingual-e5-small",
    model_type=["dense"],
    languages=XLMR_LANGUAGES,
    open_weights=True,
    revision="fd1525a9fd15316a2d503bf26ab031a61d056e98",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=118_000_000,
    n_embedding_parameters=96_014_208,
    memory_usage_mb=449,
    embed_dim=384,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/intfloat/multilingual-e5-small",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    public_training_code=None,  # couldn't find
    public_training_data=None,
    training_datasets=ME5_TRAINING_DATA,
    adapted_from="microsoft/Multilingual-MiniLM-L12-H384",
    citation=MULTILINGUAL_E5_CITATION,
)

e5_mult_base = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/multilingual-e5-base",
    model_type=["dense"],
    languages=XLMR_LANGUAGES,
    open_weights=True,
    revision="d13f1b27baf31030b7fd040960d60d909913633f",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=278_000_000,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    embed_dim=768,
    license="mit",
    max_tokens=514,
    reference="https://huggingface.co/intfloat/multilingual-e5-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    adapted_from="FacebookAI/xlm-roberta-base",
    training_datasets=ME5_TRAINING_DATA,
    citation=MULTILINGUAL_E5_CITATION,
)

e5_mult_large = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/multilingual-e5-large",
    model_type=["dense"],
    languages=XLMR_LANGUAGES,
    open_weights=True,
    revision="ab10c1a7f42e74530fe7ae5be82e6d4f11a719eb",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=560_000_000,
    n_embedding_parameters=256_002_048,
    memory_usage_mb=2136,
    embed_dim=1024,
    license="mit",
    max_tokens=514,
    reference="https://huggingface.co/intfloat/multilingual-e5-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=ME5_TRAINING_DATA,
    adapted_from="FacebookAI/xlm-roberta-large",
    citation=MULTILINGUAL_E5_CITATION,
)

e5_eng_small_v2 = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/e5-small-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="dca8b1a9dae0d4575df2bf423a5edb485a431236",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=33_000_000,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=127,
    embed_dim=384,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/intfloat/e5-small-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    adapted_from="intfloat/e5-small",
    training_datasets=E5_TRAINING_DATA,
    citation=E5_CITATION,
)

e5_eng_small = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/e5-small",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="e272f3049e853b47cb5ca3952268c6662abda68f",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=33_000_000,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=127,
    embed_dim=384,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/intfloat/e5-small",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=E5_TRAINING_DATA,
    adapted_from="sentence-transformers/all-MiniLM-L6-v2",
    citation=E5_CITATION,
)

e5_eng_base_v2 = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/e5-base-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="1c644c92ad3ba1efdad3f1451a637716616a20e8",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=109_000_000,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=418,
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/intfloat/e5-base-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    superseded_by=None,
    adapted_from="intfloat/e5-base",
    citation=E5_CITATION,
    public_training_code=None,
    public_training_data=None,
    training_datasets=E5_TRAINING_DATA,
)

e5_eng_large_v2 = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/e5-large-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="b322e09026e4ea05f42beadf4d661fb4e101d311",
    release_date=E5_PAPER_RELEASE_DATE,
    n_parameters=335_000_000,
    n_embedding_parameters=31_254_528,
    memory_usage_mb=1278,
    embed_dim=1024,
    license="mit",
    max_tokens=514,
    reference="https://huggingface.co/intfloat/e5-large-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "ONNX", "safetensors"],
    use_instructions=True,
    superseded_by=None,
    adapted_from="intfloat/e5-large",
    public_training_code=None,
    public_training_data=None,
    training_datasets=E5_TRAINING_DATA,
    citation=E5_CITATION,
)

e5_large = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/e5-large",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="4dc6d853a804b9c8886ede6dda8a073b7dc08a81",
    release_date="2022-12-26",
    n_parameters=335_000_000,
    n_embedding_parameters=31_254_528,
    memory_usage_mb=1278,
    embed_dim=1024,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/intfloat/e5-large",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    superseded_by="intfloat/e5-large-v2",
    adapted_from="google-bert/bert-large-uncased-whole-word-masking",
    public_training_code=None,
    public_training_data=None,
    training_datasets=E5_TRAINING_DATA,
    citation=E5_CITATION,
)

e5_base = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts=model_prompts,
    ),
    name="intfloat/e5-base",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="b533fe4636f4a2507c08ddab40644d20b0006d6a",
    release_date="2022-12-26",
    n_parameters=109_000_000,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=418,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/intfloat/e5-base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    superseded_by="intfloat/e5-base-v2",
    adapted_from="google-bert/bert-base-uncased",
    public_training_code=None,
    public_training_data=None,
    training_datasets=E5_TRAINING_DATA,
    citation=E5_CITATION,
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, TypedDict, cast

from scipy.stats import pearsonr, spearmanr

from mteb._evaluators import AnySTSEvaluator
from mteb.models import EncoderProtocol
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

from ._statistics_calculation import (
    calculate_audio_statistics,
    calculate_image_statistics,
    calculate_score_statistics,
    calculate_text_statistics,
)
from .abstask import AbsTask

if TYPE_CHECKING:
    from pathlib import Path

    from datasets import Dataset

    from mteb._evaluators.any_sts_evaluator import STSEvaluatorScores
    from mteb.models import MTEBModels
    from mteb.types import EncodeKwargs, PromptType
    from mteb.types.statistics import (
        AudioStatistics,
        ImageStatistics,
        ScoreStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


class AnySTSDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for STS

    Attributes:
        num_samples: number of samples in the dataset.
        number_of_characters: Total number of symbols in the dataset.
        unique_pairs: Number of unique pairs

        text1_statistics: Statistics for sentence1
        text2_statistics: Statistics for sentence2

        image1_statistics: Statistics for image1
        image2_statistics: Statistics for image2

        audio1_statistics: Statistics for audio1
        audio2_statistics: Statistics for audio2

        label_statistics: Statistics for labels
    """

    num_samples: int
    number_of_characters: int | None
    unique_pairs: int | None

    text1_statistics: TextStatistics | None
    text2_statistics: TextStatistics | None

    image1_statistics: ImageStatistics | None
    image2_statistics: ImageStatistics | None

    audio1_statistics: AudioStatistics | None
    audio2_statistics: AudioStatistics | None

    label_statistics: ScoreStatistics


class STSMetrics(TypedDict):
    """Metrics for STS tasks

    Attributes:
        pearson: Pearson correlation coefficient using the model's similarity function or cosine similarity if not available
        spearman: Spearman correlation coefficient using the model's similarity function or cosine similarity if not available
        cosine_pearson: Pearson correlation coefficient using cosine similarity
        cosine_spearman: Spearman correlation coefficient using cosine similarity
        manhattan_pearson: Pearson correlation coefficient using Manhattan distance
        manhattan_spearman: Spearman correlation coefficient using Manhattan distance
        euclidean_pearson: Pearson correlation coefficient using Euclidean distance
        euclidean_spearman: Spearman correlation coefficient using Euclidean distance
    """

    pearson: float
    spearman: float
    cosine_pearson: float
    cosine_spearman: float
    manhattan_pearson: float
    manhattan_spearman: float
    euclidean_pearson: float
    euclidean_spearman: float


class AbsTaskSTS(AbsTask):
    """The class which semantic textual similarity (STS) tasks inherit from.

    A semantic textual similarity (STS) task consists of a dataset with pairs of sentences and corresponding similarity scores.
    The task is to predict the similarity score for each pair of sentences.

    The task works by encoding the sentences using the provided model and then calculating similarity scores using both the model-defined similarity
    function (if available) and generic similarity functions, including cosine similarity, Manhattan distance, and Euclidean distance.
    The predicted similarity scores are then compared to the true similarity scores using Pearson and Spearman correlation coefficients.


    Attributes:
        dataset: Dataset or dict of Datasets for different subsets (e.g., languages). Dataset must contain columns specified in column_names and a 'score' column.
            Columns in column_names should contain the text or image data to be compared.
        column_names: Tuple containing the names of the two columns to compare.
        min_score: Minimum possible score in the dataset.
        max_score: Maximum possible score in the dataset.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
        input1_prompt_type: Type of prompt of first input. Used for asymmetric tasks.
        input2_prompt_type: Type of prompt of second input. Used for asymmetric tasks.
    """

    abstask_prompt = "Retrieve semantically similar text."
    column_names: tuple[str, str] = ("sentence1", "sentence2")
    min_score: int = 0
    max_score: int = 5
    input1_prompt_type: PromptType | None = None
    input2_prompt_type: PromptType | None = None

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: Dataset,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> STSMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        normalized_scores = list(map(self._normalize, data_split["score"]))
        data_split = data_split.select_columns(list(self.column_names))

        evaluator = AnySTSEvaluator(
            data_split,
            self.column_names,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            input1_prompt_type=self.input1_prompt_type,
            input2_prompt_type=self.input2_prompt_type,
            **kwargs,
        )
        scores = evaluator(
            model,
            encode_kwargs=encode_kwargs,
            num_proc=num_proc,
        )

        if prediction_folder:
            self._save_task_predictions(
                scores,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        return self._calculate_scores(scores, normalized_scores)

    def _calculate_scores(
        self, scores: STSEvaluatorScores, normalized_scores: list[float]
    ) -> STSMetrics:
        def compute_corr(x: list[float], y: list[float]) -> tuple[float, float]:
            """Return (pearson, spearman) correlations between x and y."""
            return float(pearsonr(x, y)[0]), float(spearmanr(x, y)[0])

        cosine_pearson, cosine_spearman = compute_corr(
            normalized_scores, scores["cosine_scores"]
        )
        manhattan_pearson, manhattan_spearman = compute_corr(
            normalized_scores, scores["manhattan_distances"]
        )
        euclidean_pearson, euclidean_spearman = compute_corr(
            normalized_scores, scores["euclidean_distances"]
        )

        if scores["similarity_scores"] is not None:
            pearson, spearman = compute_corr(
                normalized_scores, scores["similarity_scores"]
            )
        else:
            # if model does not have a similarity function, assume cosine similarity
            pearson, spearman = cosine_pearson, cosine_spearman

        return STSMetrics(
            # using the models own similarity score
            pearson=pearson,
            spearman=spearman,
            # generic similarity scores
            cosine_pearson=cosine_pearson,
            cosine_spearman=cosine_spearman,
            manhattan_pearson=manhattan_pearson,
            manhattan_spearman=manhattan_spearman,
            euclidean_pearson=euclidean_pearson,
            euclidean_spearman=euclidean_spearman,
        )

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> AnySTSDescriptiveStatistics:
        first_column, second_column = self.column_names
        self.dataset = cast("dict[str, dict[str, Dataset]]", self.dataset)

        if hf_subset:
            sentence1 = self.dataset[hf_subset][split][first_column]
            sentence2 = self.dataset[hf_subset][split][second_column]
            score = self.dataset[hf_subset][split]["score"]
        elif compute_overall:
            sentence1 = []
            sentence2 = []
            score = []
            for hf_subset in self.metadata.eval_langs:
                sentence1.extend(self.dataset[hf_subset][split][first_column])
                sentence2.extend(self.dataset[hf_subset][split][second_column])
                score.extend(self.dataset[hf_subset][split]["score"])
        else:
            sentence1 = self.dataset[split][first_column]
            sentence2 = self.dataset[split][second_column]
            score = self.dataset[split]["score"]

        if "text" in self.metadata.modalities:
            text1_statistics = calculate_text_statistics(sentence1)
            text2_statistics = calculate_text_statistics(sentence2)

            unique_pairs = len(set(zip(sentence1, sentence2)))
        else:
            text1_statistics = None
            text2_statistics = None
            unique_pairs = None

        if "image" in self.metadata.modalities:
            image1_statistics = calculate_image_statistics(sentence1)
            image2_statistics = calculate_image_statistics(sentence2)
        else:
            image1_statistics = None
            image2_statistics = None

        if "audio" in self.metadata.modalities:
            audio1_statistics = calculate_audio_statistics(sentence1)
            audio2_statistics = calculate_audio_statistics(sentence2)
        else:
            audio1_statistics = None
            audio2_statistics = None

        labels_statistics = calculate_score_statistics(score)

        return AnySTSDescriptiveStatistics(
            num_samples=len(sentence1),
            number_of_characters=(
                text1_statistics["total_text_length"]
                + text2_statistics["total_text_length"]
                if text1_statistics
                else None
            ),
            unique_pairs=unique_pairs,
            text1_statistics=text1_statistics,
            text2_statistics=text2_statistics,
            image1_statistics=image1_statistics,
            image2_statistics=image2_statistics,
            audio1_statistics=audio1_statistics,
            audio2_statistics=audio2_statistics,
            label_statistics=labels_statistics,
        )

    def _push_dataset_to_hub(
        self,
        repo_name: str,
        num_proc: int | None = None,
    ) -> None:
        self._upload_dataset_to_hub(
            repo_name,
            [self.column_names[0], self.column_names[1], "score"],
            num_proc=num_proc,
        )

    def _normalize(self, x: float) -> float:
        return (x - self.min_score) / (self.max_score - self.min_score)



from .apps_retrieval import AppsRetrieval
from .code1_retrieval import Code1Retrieval
from .code_edit_search_retrieval import CodeEditSearchRetrieval
from .code_feedback_mt_retrieval import CodeFeedbackMT
from .code_feedback_st_retrieval import CodeFeedbackST
from .code_rag import (
    CodeRAGLibraryDocumentationSolutionsRetrieval,
    CodeRAGOnlineTutorialsRetrieval,
    CodeRAGProgrammingSolutionsRetrieval,
    CodeRAGStackoverflowPostsRetrieval,
)
from .code_search_net_cc_retrieval import CodeSearchNetCCRetrieval
from .code_search_net_retrieval import CodeSearchNetRetrieval
from .code_trans_ocean_contest_retrieval import CodeTransOceanContestRetrieval
from .code_trans_ocean_dl_retrieval import CodeTransOceanDLRetrieval
from .coir_code_search_net_retrieval import COIRCodeSearchNetRetrieval
from .cos_qa_retrieval import CosQARetrieval
from .ds1000_retrieval import DS1000Retrieval
from .fresh_stack_retrieval import FreshStackRetrieval
from .human_eval_retrieval import HumanEvalRetrieval
from .japanese_code1_retrieval import JapaneseCode1Retrieval
from .mbpp_retrieval import MBPPRetrieval
from .stack_overflow_qa_retrieval import StackOverflowQARetrieval
from .synthetic_text2_sql_retrieval import SyntheticText2SQLRetrieval
from .wiki_sql_retrieval import WikiSQLRetrieval

__all__ = [
    "AppsRetrieval",
    "COIRCodeSearchNetRetrieval",
    "Code1Retrieval",
    "CodeEditSearchRetrieval",
    "CodeFeedbackMT",
    "CodeFeedbackST",
    "CodeRAGLibraryDocumentationSolutionsRetrieval",
    "CodeRAGOnlineTutorialsRetrieval",
    "CodeRAGProgrammingSolutionsRetrieval",
    "CodeRAGStackoverflowPostsRetrieval",
    "CodeSearchNetCCRetrieval",
    "CodeSearchNetRetrieval",
    "CodeTransOceanContestRetrieval",
    "CodeTransOceanDLRetrieval",
    "CosQARetrieval",
    "DS1000Retrieval",
    "FreshStackRetrieval",
    "HumanEvalRetrieval",
    "JapaneseCode1Retrieval",
    "MBPPRetrieval",
    "StackOverflowQARetrieval",
    "SyntheticText2SQLRetrieval",
    "WikiSQLRetrieval",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class GLDv2I2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="GLDv2I2IRetrieval",
        description="Retrieve names of landmarks based on their image.",
        reference="https://openaccess.thecvf.com/content_CVPR_2020/html/Weyand_Google_Landmarks_Dataset_v2_-_A_Large-Scale_Benchmark_for_Instance-Level_CVPR_2020_paper.html",
        dataset={
            "path": "mteb/gld-v2",
            "revision": "04ec40ce10b27c291cbc329045ecdfd01e06c4a7",
        },
        type="Any2AnyRetrieval",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2017-01-01", "2017-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{Weyand_2020_CVPR,
  author = {Weyand, Tobias and Araujo, Andre and Cao, Bingyi and Sim, Jack},
  booktitle = {Proceedings of the IEEE/CVF Conference on Computer Vision and Pattern Recognition (CVPR)},
  month = {June},
  title = {Google Landmarks Dataset v2 - A Large-Scale Benchmark for Instance-Level Recognition and Retrieval},
  year = {2020},
}
""",
    )
